/***************************************************************************/
/* The Minstrel's Song is (c) 1998-2005 by Jonathan Hayward.  See the file */
/* lib/copyrigh.txt for a copyright and license statement.                 */
/***************************************************************************/

#include "tms.h"
#ifndef MAC_68k
#include <sys/types.h>
#endif
#ifdef TMS_CURSES
#include <curses.h>
#endif
#include <stdio.h>
#include <stdlib.h>

#ifdef CODE_SEGMENT
#pragma segment INIT_C
#endif

struct clist *master_clist;
struct cursor_struct *cursor;
struct small_creature *player;
struct ilist *master_ilist;
struct game_variables_struct *game_variables = NULL;
struct level *current_level, *countryside, *main_city, *prev_level;
struct llist *master_llist;
struct master_creature **master_creatures;
struct master_item *master_items[NUM_ITEMS][3];
struct master_item **orig_items;
struct pointer_struct *master_pointers;
struct queue_struct *keyboard_input;
struct request_queue_struct *user_requests;
char *screen_image;
long *screen_image_changed;
long initial_user_selection;
long armor[NUM_DIST_SLOTS][3];
long device[NUM_DIST_SLOTS][3];
long furniture[NUM_DIST_SLOTS][3];
long liquid[NUM_DIST_SLOTS][3];
long natural[NUM_DIST_SLOTS][3];
long non_device_equipment[NUM_DIST_SLOTS][3];
long non_device_treasure[NUM_DIST_SLOTS][3];
long num_master_creatures;
long supplies[NUM_DIST_SLOTS][3];
long weapon[NUM_DIST_SLOTS][3];
long building_item[NUM_DIST_SLOTS][3];
long cavern_item[NUM_DIST_SLOTS][3];
long outdoors_item[NUM_DIST_SLOTS][3];
long tiny_item[NUM_DIST_SLOTS][3];

char *gender_choice[2] = {
  "Male",
  "Female"
  };

char *initial_menu_choice[6] = {
  "Have a character generated for you.",
  "Play yourself.",
  "Specify attributes.",
  "Create a new game with a saved character.",
  "Load a saved game.",
  "Exit."
  };

char *race_choice_1[7] = {
  "Janra",
  "Jec",
  "Nor'krin",
  "Shal",
  "Tuz",
  "Urvanovestilli",
  "Yedidia"
  };

char* save_filename;

void init()
    {
    long i;
    init_common();
    master_clist = NULL;
    master_ilist = NULL;
    master_llist = NULL;
    master_creatures = (struct master_creature**) calloc(NUM_CREATURES,
      sizeof(struct master_creature *));
    orig_items = (struct master_item**) calloc(NUM_ITEMS, sizeof(struct
      master_item *));
    for(i = 0; i < NUM_CREATURES; ++i)
        master_creatures[i] = &master_creatures_original[i];
    for(i = 0; i < NUM_ITEMS; ++i)
        orig_items[i] = &master_items_original[i];
    game_variables=(struct game_variables_struct *) 
      calloc(1, sizeof(struct game_variables_struct));
    game_variables->me = game_variables;
    game_variables->elapsed_units=(DAY/2);
    game_variables->elapsed_days=(300 + (ltrandom() % 1400)) * YEAR +
      (ltrandom() % YEAR);
    game_variables->flags=DURING_INIT|DAYLIGHT|TWILIGHT|MAX_REFRESH|
      MESSAGES_PROMPT|FREE_UNNEEDED_LEVELS;
    game_variables->indent_width = INDENT_DEFAULT;
    game_variables->messages_printed = 0;
    game_variables->fs_messages_printed = 0;
    game_variables->quests_completed = 0;
    game_variables->current_quest = SENT_ON_QUEST;
    game_variables->scorpion_stings_found = 0;
    game_variables->save_file_compression_level = 6;
#ifdef GTK
    game_variables->window_dungeon_columns = 31;
    game_variables->window_dungeon_lines = 21;
#endif
    check_files();
    for(i=0;i<NUM_ITEMS;++i)
        {
        master_items[i][MATRIX_NORMAL] = (struct master_item *)
            calloc(1, sizeof(struct master_item));
        master_items[i][MATRIX_FINE] = (struct master_item *)
            calloc(1, sizeof(struct master_item));
        master_items[i][MATRIX_POOR] = (struct master_item *)
            calloc(1, sizeof(struct master_item));
        *master_items[i][MATRIX_NORMAL] = *orig_items[i];
        *master_items[i][MATRIX_FINE] = *orig_items[i];
        make_fine(master_items[i][MATRIX_FINE]);
        *master_items[i][MATRIX_POOR] = *orig_items[i];
        make_poor(master_items[i][MATRIX_POOR]);
        }
#ifdef GTK
    init_machdep();
#else
    window_refresh();
    noecho();
    cbreak();
#endif
        /*
    if (window_lines() < 24 || window_columns() < 80)
        {
        perror("Error: insufficient terminal size.  The Minstrel's Song needs at least 24 rows\nand 80 columns.");
        my_finish(1);
        }
    read_file(FILE_MOTD);
    */
    window_clear();
#ifndef GTK
    scrollok(stdscr, 1);
    top_text=subwin(stdscr, 6, window_columns(), 0, 0);
    scrollok(top_text, 1);
    left_panel=subwin(stdscr, 18, 11, 6, 0);
    bottom_panel=subwin(stdscr, 3, window_columns()-14, window_lines()-3, 13);
    dungeon=subwin(stdscr, window_lines()-10, window_columns()-16, 6, 13);
    puzzle=subwin(stdscr, window_lines()-7, window_columns(), 6, 0);
#endif
#ifdef MAC_68k
    flag_set(&game_variables->flags, ASK_PICKUP|FREE_UNNEEDED_LEVELS|
      ALWAYS_DISPLAY_STAIRCASES);
#else
    flag_set(&game_variables->flags, PLAYER_AT_CENTER|ASK_PICKUP|
      FREE_UNNEEDED_LEVELS|ALWAYS_DISPLAY_STAIRCASES);
#endif
    generate_player();
    pmsg("Welcome to The Minstrel's Song.");
    pmsg("(Please wait one moment, as I prepare things for you...)");
    pmsg("");
    window_refresh();
    current_level = make_level(LEVEL_COUNTRYSIDE, 0, LOAD, 0);
    /*
    current_level = make_level(LEVEL_DUNGEON, 0, 0, 0);
    */
    no_hostile_add_creature(current_level, player,
      game_variables->new_level_player_x, game_variables->new_level_player_y);
    change_level(player->x, player->y);
    main_city = current_level;
    game_variables->messages_printed = 0;
    calc_ready(player);
    pmsg("You arrive at a great Urvanovestilli city, where you have heard \
there is a");
    nocap_pmsg("mentor who will help you on your quests.");
#ifdef GTK
    window_refresh();
#else
    touchwin(stdscr);
    update_window();
    pmsg("");
    morewait();
#endif
#ifdef GTK
    pmsg("Hint:");
#else
    pmsg("Would you like a hint about the game?");
    if (confirm_silent())
#endif
        hint();
#ifndef GTK
    pmsg("");
    pmsg("Hit '?' for help.");
#endif
    flag_reset(&game_variables->flags, DURING_INIT);
    }

void init_restore(char *given_save_filename)
    {
    long i;
    init_common();
#ifndef GTK
    initscr();
#endif
    master_creatures = (struct master_creature**) calloc(NUM_CREATURES,
      sizeof(struct master_creature *));
    orig_items = (struct master_item**) calloc(NUM_ITEMS, sizeof(struct
      master_item *));
    for(i = 0; i < NUM_CREATURES; ++i)
    for(i = 0; i < NUM_CREATURES; ++i)
        master_creatures[i] = &master_creatures_original[i];
    for(i = 0; i < NUM_ITEMS; ++i)
        orig_items[i] = &master_items_original[i];
    save_filename = given_save_filename;
    master_clist = NULL;
    master_ilist = NULL;
    master_llist = NULL;
    game_variables=(struct game_variables_struct *) 
      calloc(1, sizeof(struct game_variables_struct));
    game_variables->me = game_variables;
    game_variables->flags=DURING_INIT;
    game_variables->indent_width = INDENT_DEFAULT;
    game_variables->messages_printed = 0;
    game_variables->fs_messages_printed = 0;
    check_files();
    for(i=0;i<NUM_ITEMS;++i)
        {
        master_items[i][MATRIX_NORMAL] = (struct master_item *)
          calloc(1, sizeof(struct master_item));
        master_items[i][MATRIX_FINE] = (struct master_item *)
          calloc(1, sizeof(struct master_item));
        master_items[i][MATRIX_POOR] = (struct master_item *)
          calloc(1, sizeof(struct master_item));
        *master_items[i][MATRIX_NORMAL] = *orig_items[i];
        *master_items[i][MATRIX_FINE] = *orig_items[i];
        make_fine(master_items[i][MATRIX_FINE]);
        *master_items[i][MATRIX_POOR] = *orig_items[i];
        make_poor(master_items[i][MATRIX_POOR]);
        }
        /*
    if (window_lines() < 24 || window_columns() < 80)
        {
        perror("Error: insufficient terminal size.  The Minstrel's Song needs at least 24 rows\nand 80 columns.");
        my_finish(1);
        }
    read_file(FILE_MOTD);
    */
    window_refresh();
#ifndef GTK
    scrollok(stdscr, 1);
    top_text=subwin(stdscr, 6, window_columns(), 0, 0);
    scrollok(top_text, 1);
    left_panel=subwin(stdscr, 18, 11, 6, 0);
    bottom_panel=subwin(stdscr, 3, window_columns()-14, window_lines()-3, 13);
    dungeon=subwin(stdscr, window_lines()-10, window_columns()-16, 6, 13);
    puzzle=subwin(stdscr, window_lines()-7, window_columns(), 6, 0);
#endif
#ifdef GTK
    flag_set(&game_variables->flags, ASK_PICKUP|MESSAGES_PROMPT);
#else
    flag_set(&game_variables->flags, PLAYER_AT_CENTER|ASK_PICKUP|
      MESSAGES_PROMPT);
#endif
    restore(given_save_filename);
    calc_ready(player);
    flag_reset(&game_variables->flags, DURING_INIT);
    refresh_screen();
    }

void generate_player(void)
    {
    char scratch[256], scratch2[256], my_save_filename[256];
    char resp;
    long age, age_class, race, gender, i;
#ifdef GTK
    /*
    for(resp = 'e'; ((resp < 'a') || (resp > 'd')); resp = new_tgetc("Welcome to the world Espiriticthus. Would you like to:\n\n\ta: have a character generated for you,\n\tb: play yourself,\n\tc: specify your character's attributes, or \n\td: load a saved character?"))
    */
    resp = initial_user_selection;
    /*
    for(resp = 5; ((resp < 0) || (resp > 4));
      resp = menu_choice(
      "Welcome to the world Espiriticthus. Would you like to:",
      initial_menu_choices,
      4))
        my_noop();
    */
    resp += 'a';
#else
    window_clear();
    fs_pmsg("");
    fs_pmsg("Welcome to the world Espiriticthus.  Would you like to:\n");
    fs_pmsg("\n");
    fs_pmsg("\ta: have a character generated for you,\n");
    fs_pmsg("\tb: play yourself,\n");
    fs_pmsg("\tc: specify attributes, or\n");
    fs_pmsg("\td: create a new game with a saved character?\n");
    fs_pmsg("\te: load a saved game?\n");
    fs_pmsg("\n");
#ifndef MAC_68k
    window_refresh();
#endif
    for(resp = 'f'; ((resp < 'a') || (resp > 'e')); resp = tgetc())
        my_noop();
#endif
    switch(resp)
        {
        case 'a':
#ifdef GTK
            resp = menu_choice("What race would you like to be?",
            race_choice_1, 7);
            resp += 'a';
#else
            fs_pmsg("");
            fs_pmsg("What race would you like to be?\n");
            fs_pmsg("\ta: Janra\n");
            fs_pmsg("\tb: Jec\n");
            fs_pmsg("\tc: Nor'krin\n");
            fs_pmsg("\td: Shal\n");
            fs_pmsg("\te: Tuz\n");
            fs_pmsg("\tf: Urvanovestilli\n");
            fs_pmsg("\tg: Yedidia\n");
#ifndef MAC_68k
            window_refresh();
#endif
            for(resp=tgetc(); (resp < 'a' || resp > 'g'); resp=tgetc())
#endif
                my_noop();
            race=resp-'a';
#ifdef GTK
            resp = menu_choice("What gender would you like to be?",
              gender_choice, 2);
            resp += 'a';
#else
            fs_pmsg("");
            fs_pmsg("What gender would you like to be?\n");
            fs_pmsg("\ta: male\n");
            fs_pmsg("\tb: female\n");
#ifndef MAC_68k
            window_refresh();
#endif
            for(resp=tgetc(); (resp < 'a' || resp > 'b'); resp=tgetc())
#endif
                my_noop();
            gender=resp-'a';
            fs_pmsg("");
#ifdef GTK
            sprintf(scratch,"And what age would you line to be\n\t(%ld-%ld)",age_class_years[race][CHILD]/2,age_class_years[race][NUM_AGE_CLASSES-1]);
            new_tgets(scratch, scratch2, NULL);
            for(age = strtol(scratch2, NULL, 10); (age <
              age_class_years[race][CHILD]/2 || age >
              age_class_years[race][NUM_AGE_CLASSES-1]);)
                {
                new_tgets(scratch, scratch2, NULL);
                age = strtol(scratch2, NULL, 10);
                }
#else
            fs_pmsg("And what age would you like to be?\n");
            sprintf(scratch,"\t(%ld-%ld)\n",age_class_years[race][CHILD]/2,age_class_years[race][NUM_AGE_CLASSES-1]);
            fs_pmsg(scratch);
            window_refresh();
            echo();
            for(scanw("%ld",&age); (age < age_class_years[race][CHILD]/2
              || age > age_class_years[race][NUM_AGE_CLASSES-1]);
                scanf("%ld",&age));
            noecho();
#endif
            window_clear();
            age_class=0;
            if (age > age_class_years[race][0]) age_class=1;
            if (age > age_class_years[race][1]) age_class=2;
            if (age > age_class_years[race][2]) age_class=3;
            if (age > age_class_years[race][3]) age_class=4;
            if (age > age_class_years[race][4]) age_class=5;
            player=extended_creature_creature_create(race, age_class, gender);
            player->age=age;
            player->extended_creature->age=age;
            player->extended_creature->unspent_ep=100;
            strncpy(player->extended_creature->title, "you", 255);
            player->extended_creature->title[255] = '\0';
#ifdef GTK
            if (tms_confirm("Save character for future play?"))
#else
            fs_pmsg("Save character for future play?");
            window_refresh();
            if (confirm_silent_internal())
#endif
                save_player();
        break;
        case 'b':
            interview();
            player->extended_creature->unspent_ep=100;
            strncpy(player->extended_creature->title, "you", 255);
            player->extended_creature->title[255] = '\0';
            flag_set(&player->flags, BELLICOSE);
#ifdef GTK
            if (tms_confirm("Save character for future play?"))
#else
            pmsg("Save character for future play?");
            window_refresh();
            if (confirm_silent_internal())
                save_player();
#endif
            read_file(FILE_PLAYER_ENTERS_GAME);
        break;
        case 'c':
#ifdef GTK
            race = menu_choice("What race would you like to be?",
              race_choice_1, 7);
#else
            fs_pmsg("What race would you like to be?\n");
            fs_pmsg("\ta: Janra\n");
            fs_pmsg("\tb: Jec\n");
            fs_pmsg("\tc: Nor'krin\n");
            fs_pmsg("\td: Shal\n");
            fs_pmsg("\te: Tuz\n");
            fs_pmsg("\tf: Urvanovestilli\n");
            fs_pmsg("\tg: Yedidia\n");
#ifdef MAC_68k
            window_refresh();
#endif
            for(resp=tgetc(); (resp < 'a' || resp > 'g'); resp=tgetc())
#endif
                my_noop();
            race=resp-'a';
#ifdef GTK
            resp = menu_choice("What gender would you like to be?",
              gender_choice, 2);
            resp += 'a';
#else
            fs_pmsg("What gender would you like to be?\n");
            fs_pmsg("\ta: male\n");
            fs_pmsg("\tb: female\n");
            window_refresh();
            for(resp=tgetc(); (resp < 'a' || resp > 'b'); resp=tgetc())
#endif
                my_noop();
            gender=resp-'a';
#ifdef GTK
            sprintf(scratch,"And what age would you like to be?\n\t(%ld-%ld)\n",age_class_years[race][CHILD]/2,age_class_years[race][NUM_AGE_CLASSES-1]);
            new_tgets(scratch, scratch2, NULL);
            for(age = strtol(scratch2, NULL, 10); (age <
              age_class_years[race][CHILD]/2 || age >
              age_class_years[race][NUM_AGE_CLASSES-1]);)
                {
                new_tgets(scratch, scratch2, NULL);
                age = strtol(scratch2, NULL, 10);
                }
#else
            fs_pmsg("And what age would you like to be?\n");
            sprintf(scratch,"\t(%ld-%ld)\n",age_class_years[race][CHILD]/2,age_class_years[race][NUM_AGE_CLASSES-1]);
            fs_pmsg(scratch);
            window_refresh();
            echo();
            for(scanw("%ld",&age); (age < age_class_years[race][CHILD]/2
              || age > age_class_years[race][NUM_AGE_CLASSES-1]);
                scanf("%ld",&age));
            noecho();
            window_clear();
#endif
            age_class=0;
            if (age > age_class_years[race][0]) age_class=1;
            if (age > age_class_years[race][1]) age_class=2;
            if (age > age_class_years[race][2]) age_class=3;
            if (age > age_class_years[race][3]) age_class=4;
            if (age > age_class_years[race][4]) age_class=5;
            player=extended_creature_creature_create(race, age_class, gender);
#ifdef GTK
            new_tgets("Please enter attributes, unadjusted for race, age, and gender; 1 is the \nlowest, 10 the average, and 35 the absolute minimum.\n\nAbility to Learn?", scratch, NULL);
            player->extended_creature->perm_attributes[AL] = strtol(scratch,
            NULL, 10) - 10;
#else
            fs_pmsg("Please enter attributes, unadjusted for race, age, and gender; -25 is the \n");
            fs_pmsg("absolute lowest, 0 the average, and 25 the absolute highest.\n");
            printw("Ability to Learn?");
            window_refresh();
            scanw("%ld", &player->extended_creature->perm_attributes[AL]) - 10;
#endif
#ifdef GTK
            new_tgets("Agility?", scratch, NULL);
            player->extended_creature->perm_attributes[AG] = strtol(scratch,
            NULL, 10);
#else
            printw("Agility?");
            window_refresh();
            scanw("%ld", &player->extended_creature->perm_attributes[AG]) - 10;
#endif
#ifdef GTK
            new_tgets("Charisma?", scratch, NULL);
            player->extended_creature->perm_attributes[CH] = strtol(scratch,
            NULL, 10);
#else
            printw("Charisma?");
            window_refresh();
            scanw("%ld", &player->extended_creature->perm_attributes[CH]) - 10;
#endif
#ifdef GTK
            new_tgets("Constitution?", scratch, NULL);
            player->extended_creature->perm_attributes[CO] = strtol(scratch,
            NULL, 10);
#else
            printw("Constitution?");
            window_refresh();
            scanw("%ld", &player->extended_creature->perm_attributes[CO]) - 10;
#endif
#ifdef GTK
            new_tgets("Dexterity?", scratch, NULL);
            player->extended_creature->perm_attributes[DE] = strtol(scratch,
            NULL, 10) - 10;
#else
            printw("Dexterity?");
            window_refresh();
            scanw("%ld", &player->extended_creature->perm_attributes[DE]);
#endif
#ifdef GTK
            new_tgets("Intelligence?", scratch, NULL);
            player->extended_creature->perm_attributes[IN] = strtol(scratch,
            NULL, 10) - 10;
#else
            printw("Intelligence?");
            window_refresh();
            scanw("%ld", &player->extended_creature->perm_attributes[IN]);
#endif
#ifdef GTK
            new_tgets("Knowledge?", scratch, NULL);
            player->extended_creature->perm_attributes[KN] = strtol(scratch,
            NULL, 10) - 10;
#else
            printw("Knowledge?");
            window_refresh();
            scanw("%ld", &player->extended_creature->perm_attributes[KN]);
#endif
#ifdef GTK
            new_tgets("Memory?", scratch, NULL);
            player->extended_creature->perm_attributes[ME] = strtol(scratch,
            NULL, 10) - 10;
#else
            printw("Memory?");
            window_refresh();
            scanw("%ld", &player->extended_creature->perm_attributes[ME]);
#endif
#ifdef GTK
            new_tgets("Perception?", scratch, NULL);
            player->extended_creature->perm_attributes[PE] = strtol(scratch,
            NULL, 10) - 10;
#else
            printw("Perception?");
            window_refresh();
            scanw("%ld", &player->extended_creature->perm_attributes[PE]);
#endif
#ifdef GTK
            new_tgets("Speed?", scratch, NULL);
            player->extended_creature->perm_attributes[SPEED] = strtol(scratch,
            NULL, 10) - 10;
#else
            printw("Speed?");
            window_refresh();
            scanw("%ld", &player->extended_creature->perm_attributes[SPEED]);
#endif
#ifdef GTK
            new_tgets("Strength?", scratch, NULL);
            player->extended_creature->perm_attributes[ST] = strtol(scratch,
            NULL, 10) - 10;
#else
            printw("Strength?");
            window_refresh();
            scanw("%ld", &player->extended_creature->perm_attributes[ST]);
#endif
#ifdef GTK
            new_tgets("Willpower?", scratch, NULL);
            player->extended_creature->perm_attributes[WI] = strtol(scratch,
            NULL, 10) - 10;
#else
            printw("Willpower?");
            window_refresh();
            scanw("%ld", &player->extended_creature->perm_attributes[WI]);
#endif
            report_attr(player);
            for(i = 0; i < NUM_ATTRIBUTES; ++i)
                {
                player->extended_creature->perm_attributes[i] = min(
                  player->extended_creature->perm_attributes[i], 25);
                player->extended_creature->perm_attributes[i] = max(
                  player->extended_creature->perm_attributes[i], 0 - 25);
                player->extended_creature->perm_attributes[i] +=
                  racial_attribute_adjustment[race][i];
                player->extended_creature->perm_attributes[i] +=
                  gender_attribute_adjustment[gender][i];
                player->extended_creature->perm_attributes[i] +=
                  age_attribute_adjustment[age_class][i];
                }
            player->age=age;
            player->extended_creature->age=age;
            player->extended_creature->unspent_ep=100;
            flag_set(&player->flags, BELLICOSE);
            strncpy(player->extended_creature->title, "you", 255);
            player->extended_creature->title[255] = '\0';
            report_attr(player);
            calc_ready(player);
            report_attr(player);
#ifdef GTK
            if (tms_confirm("Save character for future play?"))
#else
            printw("Save character for future play?");
            window_refresh();
            if (confirm_silent_internal())
#endif
                save_player();
        break;
        case 'd':
            load_player();
        break;
        case 'e':
            g_print("Just before.\n");
            if (new_tgets("What is the filename?", my_save_filename,
              NULL))
                init_restore(my_save_filename);
            else
                exit(0);
            g_print("Just after.\n");
        break;
        }
    window_clear();
    }

void init_common()
    {
    keyboard_input = NULL;
    user_requests = NULL;
    save_filename = calloc(256, sizeof(char));
    strcpy(save_filename, SAVE_FILE);
    game_variables = (struct game_variables_struct *) calloc(1, sizeof(struct
      game_variables_struct));
    flag_set(&game_variables->flags, IN_BETWEEN_GAMES);
    cursor = (struct cursor_struct *) calloc(1, sizeof(struct cursor_struct));
    cursor->x = 0;
    cursor->y = 0;
    cursor->should_display = 0;
#ifdef GTK
    gtk_specific_init();
#endif
    }
