/***************************************************************************/
/* The Minstrel's Song is (c) 1998-2005 by Jonathan Hayward.  See the file */
/* lib/copyrigh.txt for a copyright and license statement.                 */
/***************************************************************************/

#include "tms.h"

#ifdef CODE_SEGMENT
#pragma segment LEVEL2_C
#endif

void prepare_space(struct space_struct *space, long identity, long race)
    {
    struct ilist *this_ilist;
    struct small_item *this_item;
    space->au=space->el=space->si=space->cu=0;
    space->creatures=NULL;
    space->items=NULL;
    space->special=NULL;
    space->type = 0;
    space->flags=0;
    space->property = NONE;
    space->room_description = ST_EMPTY;
    space->sign = ST_EMPTY;
    switch(identity)
        {
        case S_ALTAR:
            flag_set(&space->flags, TRANSPARENT);
            space->type=L_ALTAR;
            space->icon=ICON_ALTAR;
            space->space_description = ST_AN_ALTAR;
        break;
        case S_AUTOMATIC_EXIT:
            flag_set(&space->flags, CAN_OCCUPY|AUTOMATIC_EFFECT|TRANSPARENT);
            space->type=L_FLOOR;
            space->icon=ICON_FLOOR;
            space->property=P_EXIT;
            space->space_description = ST_THE_GROUND;
            space->special = new_special();
        break;
        case S_BED:
            flag_set(&space->flags, CAN_OCCUPY|TRANSPARENT|AUTOMATIC_EFFECT);
            space->type = L_BED;
            space->icon=ICON_BED;
            space->property = P_BED;
            space->space_description = ST_A_BED;
        break;
        case S_CABINET:
            flag_set(&space->flags, CAN_OCCUPY|TRANSPARENT);
            space->type = L_FLOOR;
            space->icon = ICON_CABINET_SPACE;
            space->space_description = ST_A_CABINET;
        break;
        case S_CATHEDRAL_ENTRANCE:
            flag_set(&space->flags, CAN_OCCUPY|TRANSPARENT|AUTOMATIC_EFFECT);
            space->type=L_ENTRANCE;
            space->icon=ICON_OPEN_DOOR;
            space->property = P_ENTRANCE;
            space->space_description = ST_AN_OPEN_DOOR;
            space->sign = ST_ENTER_IN_SILENCE;
            space->special = new_special();
            space->special->level_type = LEVEL_CATHEDRAL;
            flag_set(&space->special->level_flags, LOAD);
        break;
        case S_CAVERN_ENTRANCE:
            flag_set(&space->flags, TRANSPARENT|CAN_OCCUPY);
            space->type=L_STAIRWELL_DOWN;
            space->icon=ICON_STAIRWELL_DOWN;
            space->property = P_STAIRWELL_DOWN;
            space->space_description = ST_OUTSIDE_THE_MOUTH_OF_A_CAVERN;
            space->special = new_special();
            space->special->level_type = LEVEL_SMALL_CAVERN;
            space->special->animal_encounter_type = NONE;
            space->special->animal_encounter_frequency = NONE;
        break;
        case S_CHAIR:
            flag_set(&space->flags, TRANSPARENT|CAN_OCCUPY);
            space->type = L_FLOOR;
            space->icon=ICON_CHAIR_SPACE;
            space->space_description = ST_A_CHAIR;
        break;
        case S_CLOCKWORK_DOOR:
            flag_set(&space->flags, DOOR_CLOSED|DOOR_LOCKED);
            space->type = L_CLOCKWORK_DOOR;
            space->icon = ICON_CLOSED_DOOR;
            space->property = P_CLOCKWORK_DOOR;
            switch(trandom() % 3)
                {
                case 0:
                    switch(trandom() % 5)
                        {
                        case 0:
                            space->other = PUZZLE_BOARD_COMPLETE_SQUARE;
                        break;
                        case 1:
                            space->other = PUZZLE_BOARD_CONVERT_ADJACENT;
                        break;
                        case 2:
                            space->other = PUZZLE_BOARD_HIDDEN_MAZE;
                        break;
                        case 3:
                            space->other = PUZZLE_BOARD_RANDOM;
                        break;
                        case 4:
                            space->other = PUZZLE_BOARD_SURROUND_OPPONENT;
                        break;
                        }
                break;
                case 1:
                    switch(trandom() % 5)
                        {
                        case 0:
                            space->other = PUZZLE_PEG_ALL_BUT_LAST_DOWN;
                        break;
                        case 1:
                            space->other = PUZZLE_PEG_ALTERNATING;
                        break;
                        case 2:
                            space->other = PUZZLE_PEG_CHECKSUM;
                        break;
                        case 3:
                            space->other = PUZZLE_PEG_RANDOM;
                        break;
                        case 4:
                            space->other = PUZZLE_PEG_SYMMETRY;
                        break;
                        }
                break;
                case 2:
                    space->other = PUZZLE_RIDDLE;
                    space->special = new_special();
                    space->special->text = new_cryptogram();
                break;
                }
            space->space_description = ST_A_CLOSED_DOOR;
        break;
        case S_CLOSED_DOOR:
            flag_set(&space->flags, DOOR_CLOSED);
            space->type=L_DOOR;
            space->icon=ICON_CLOSED_DOOR;
            space->space_description = ST_A_CLOSED_DOOR;
        break;
        case S_COLLECTION_BOX:
            flag_set(&space->flags, TRANSPARENT|CAN_OCCUPY|AUTOMATIC_EFFECT);
            space->property = P_COLLECTION_BOX;
            space->type=L_FLOOR;
            space->icon=ICON_MECHANICAL_SPACE;
            space->space_description = ST_A_COLLECTION_BOX;
        break;
        case S_COUNTRYSIDE_CAVERN_ENTRANCE:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY);
            space->type=L_STAIRWELL_DOWN;
            space->icon=ICON_COUNTRYSIDE_FEATURE;
            space->property = P_STAIRWELL_DOWN;
            space->space_description = ST_OUTSIDE_THE_MOUTH_OF_A_CAVERN;
            space->room_description = ST_RHOG;
            space->special = new_special();
            space->special->level_type = LEVEL_CAVERN;
            space->special->animal_encounter_type = NONE;
            space->special->animal_encounter_frequency = NONE;
        break;
        case S_COUNTRYSIDE_JEC_MIDDLE_PLAINS:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY);
            space->type=L_FLOOR;
            space->icon=ICON_COUNTRYSIDE_PLAINS;
            space->property = NONE;
            space->space_description = ST_THE_MIDDLE_PLAINS;
            space->room_description = ST_TEV;
            space->special = new_special();
            space->special->level_type = LEVEL_JEC_MIDDLE_PLAINS;
            space->special->animal_encounter_type = JEC_ENCOUNTER;
            space->special->animal_encounter_frequency = 150;
            space->special->human_encounter_type = JEC;
            space->special->human_encounter_frequency = 1000;
        break;
        case S_COUNTRYSIDE_JEC_TOWN:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY|TRANSPARENT);
            space->type=L_FLOOR;
            space->icon=ICON_COUNTRYSIDE_SETTLEMENT;
            space->property = P_PUSHES_LEVEL;
            space->space_description = ST_OUTSIDE_A_JEC_TOWN;
            space->room_description = ST_TEV;
            space->special = new_special();
            space->special->level_type = LEVEL_JEC_TOWN;
        break;
        case S_COUNTRYSIDE_NORKRIN_EAST_RIDGE_MOUNTAINS:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY|AUTOMATIC_EFFECT|IS_COLD);
            space->type=L_FLOOR;
            space->icon=ICON_COUNTRYSIDE_MOUNTAINS;
            space->property = P_PEAK;
            space->space_description = ST_THE_EAST_RIDGE_MOUNTAINS;
            space->room_description = ST_CRYONA;
            space->special = new_special();
            space->special->level_type = LEVEL_NORKRIN_EAST_RIDGE_MOUNTAINS;
            space->special->animal_encounter_type = NORKRIN_ENCOUNTER;
            space->special->animal_encounter_frequency = 200;
            space->special->human_encounter_type = NORKRIN;
            space->special->human_encounter_frequency = 2000;
        break;
        case S_COUNTRYSIDE_NORKRIN_ICE_PEAKS:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY|AUTOMATIC_EFFECT|IS_COLD|EXTREMELY_COLD);
            space->type=L_PEAK;
            space->icon=ICON_COUNTRYSIDE_MOUNTAINS;
            space->property = P_PEAK;
            space->space_description = ST_THE_ICE_PEAKS;
            space->room_description = ST_CRYONA;
            space->special = new_special();
            space->special->level_type = LEVEL_NORKRIN_ICE_PEAKS;
            space->special->animal_encounter_type = NORKRIN_ENCOUNTER;
            space->special->animal_encounter_frequency = 200;
            space->special->human_encounter_type = NORKRIN;
            space->special->human_encounter_frequency = 2000;
        break;
        case S_COUNTRYSIDE_NORKRIN_ICE_PEAKS_HIDDEN_POOL:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY|AUTOMATIC_EFFECT|HAS_SECRET);
            space->type=L_PEAK;
            space->icon=ICON_COUNTRYSIDE_MOUNTAINS;
            space->property = P_PEAK;
            space->space_description = ST_THE_ICE_PEAKS;
            space->room_description = ST_CRYONA;
            space->other = S_COUNTRYSIDE_NORKRIN_ICE_PEAKS_POOL;
            space->special = new_special();
            space->special->level_type = LEVEL_NORKRIN_ICE_PEAKS;
            space->special->animal_encounter_type = NORKRIN_ENCOUNTER;
            space->special->animal_encounter_frequency = 200;
            space->special->human_encounter_type = NORKRIN;
            space->special->human_encounter_frequency = 2000;
        break;
        case S_COUNTRYSIDE_NORKRIN_ICE_PEAKS_POOL:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY);
            space->type=L_ENTRANCE;
            space->icon=ICON_COUNTRYSIDE_FEATURE;
            space->property = P_ENTRANCE;
            space->space_description = ST_OUTSIDE_A_POOL_OF_WATER;
            space->room_description = ST_CRYONA;
            space->special = new_special();
            space->special->level_type = LEVEL_NORKRIN_ICE_PEAKS_HIDDEN_POOL;
            flag_set(&space->special->level_flags, LOAD);
        break;
        case S_COUNTRYSIDE_NORKRIN_PLAINS:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY|IS_COLD);
            space->type=L_FLOOR;
            space->icon=ICON_COUNTRYSIDE_PLAINS;
            space->space_description = ST_THE_NORTHERN_WASTES;
            space->room_description = ST_CRYONA;
            space->property = NONE;
            space->special = new_special();
            space->special->level_type = LEVEL_NORKRIN_PLAINS;
            space->special->animal_encounter_type = NORKRIN_ENCOUNTER;
            space->special->animal_encounter_frequency = 100;
            space->special->human_encounter_type = NORKRIN;
            space->special->human_encounter_frequency = 1000;
        break;
        case S_COUNTRYSIDE_OCEAN:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY|IS_WATER|SALT_WATER|AUTOMATIC_EFFECT);
            space->type=L_WATER;
            space->icon=ICON_COUNTRYSIDE_OCEAN;
            space->property = P_DEEP_WATER;
            space->space_description = ST_THE_OCEAN;
            space->special = new_special();
            space->special->level_type = LEVEL_OCEAN;
            space->special->animal_encounter_type = SEA_ENCOUNTER;
            space->special->animal_encounter_frequency = 150;
        break;
        case S_COUNTRYSIDE_URVANOVESTILLI_CITY:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY);
            space->type=L_FLOOR;
            space->icon=ICON_COUNTRYSIDE_SETTLEMENT;
            space->property = NONE;
            space->space_description = ST_OUTSIDE_AN_URVANOVESTILLI_CITY;
            space->room_description = ST_FLARISTIMMO;
            space->special = new_special();
            space->special->level_type = LEVEL_URVANOVESTILLI_CITY;
        break;
        case S_COUNTRYSIDE_URVANOVESTILLI_EAST_RIDGE_MOUNTAINS:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY|AUTOMATIC_EFFECT);
            space->type=L_PEAK;
            space->icon=ICON_COUNTRYSIDE_MOUNTAINS;
            space->property = P_PEAK;
            space->space_description = ST_THE_EAST_RIDGE_MOUNTAINS;
            space->room_description = ST_FLARISTIMMO;
            space->special = new_special();
            space->special->level_type = LEVEL_URVANOVESTILLI_EAST_RIDGE_MOUNTAINS;
            space->special->animal_encounter_type = URVANOVESTILLI_ENCOUNTER;
            space->special->animal_encounter_frequency = 100;
            space->special->human_encounter_type = URVANOVESTILLI;
            space->special->human_encounter_frequency = 500;
        break;
        case S_COUNTRYSIDE_URVANOVESTILLI_FOG_VALLEY:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY);
            space->type=L_FLOOR;
            space->icon=ICON_COUNTRYSIDE_PLAINS;
            space->property = NONE;
            space->space_description = ST_THE_FOG_VALLEY;
            space->room_description = ST_FLARISTIMMO;
            space->special = new_special();
            space->special->level_flags = L_CLOAKED_BY_MIST;
            space->special->level_type = LEVEL_URVANOVESTILLI_FOG_VALLEY;
            space->special->animal_encounter_type = URVANOVESTILLI_ENCOUNTER;
            space->special->animal_encounter_frequency = 100;
            space->special->human_encounter_type = URVANOVESTILLI;
            space->special->human_encounter_frequency = 500;
        break;
        case S_COUNTRYSIDE_URVANOVESTILLI_HIDDEN_MANSION:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY|HAS_SECRET);
            space->type=L_FLOOR;
            space->icon=ICON_COUNTRYSIDE_PLAINS;
            space->property = NONE;
            space->space_description = ST_THE_MIDDLE_PLAINS;
            space->room_description = ST_FLARISTIMMO;
            space->other = S_COUNTRYSIDE_URVANOVESTILLI_MANSION;
            space->special = new_special();
            space->special->level_type = LEVEL_URVANOVESTILLI_PLAINS;
            space->special->animal_encounter_type = URVANOVESTILLI_ENCOUNTER;
            space->special->animal_encounter_frequency = 72;
        break;
        case S_COUNTRYSIDE_URVANOVESTILLI_MANSION:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY);
            space->type=L_FLOOR;
            space->icon=ICON_COUNTRYSIDE_FEATURE;
            space->property = NONE;
            space->space_description = ST_THE_MIDDLE_PLAINS;
            space->room_description = ST_FLARISTIMMO;
            space->special = new_special();
            space->special->level_type = LEVEL_URVANOVESTILLI_MANSION;
        break;
        case S_COUNTRYSIDE_URVANOVESTILLI_PLAINS:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY);
            space->type=L_FLOOR;
            space->icon=ICON_COUNTRYSIDE_PLAINS;
            space->property = NONE;
            space->space_description = ST_THE_MIDDLE_PLAINS;
            space->room_description = ST_FLARISTIMMO;
            space->special = new_special();
            space->special->level_type = LEVEL_URVANOVESTILLI_PLAINS;
            space->special->animal_encounter_type = URVANOVESTILLI_ENCOUNTER;
            space->special->animal_encounter_frequency = 72;
            space->special->human_encounter_type = URVANOVESTILLI;
            space->special->human_encounter_frequency = 500;
        break;
        case S_COUNTRYSIDE_TUZ_EAST_RIDGE_MOUNTAINS:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY|AUTOMATIC_EFFECT);
            space->type=L_PEAK;
            space->icon=ICON_COUNTRYSIDE_MOUNTAINS;
            space->property = P_PEAK;
            space->space_description = ST_THE_EAST_RIDGE_MOUNTAINS;
            space->room_description = ST_RHOG;
            space->special = new_special();
            space->special->level_type = LEVEL_TUZ_EAST_RIDGE_MOUNTAINS;
            space->special->animal_encounter_type = TUZ_ENCOUNTER;
            space->special->animal_encounter_frequency = 20;
            space->special->human_encounter_type = TUZ;
            space->special->human_encounter_frequency = 700;
        break;
        case S_COUNTRYSIDE_TUZ_FOREST:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY);
            space->type=L_TREE;
            space->icon=ICON_COUNTRYSIDE_FOREST;
            space->property = NONE;
            space->space_description = ST_A_DENSE_FOREST;
            space->room_description = ST_RHOG;
            space->special = new_special();
            space->special->level_type = LEVEL_TUZ_FOREST;
            space->special->animal_encounter_type = TUZ_ENCOUNTER;
            space->special->animal_encounter_frequency = 36;
            space->special->human_encounter_type = TUZ;
            space->special->human_encounter_frequency = 700;
        break;
        case S_COUNTRYSIDE_TUZ_VILLAGE:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY);
            space->type=L_FLOOR;
            space->icon=ICON_COUNTRYSIDE_SETTLEMENT;
            space->property = NONE;
            space->space_description = ST_OUTSIDE_A_SMALL_TUZ_VILLAGE;
            space->room_description = ST_RHOG;
            space->special = new_special();
            space->special->level_type = LEVEL_TUZ_VILLAGE;
        break;
        case S_COUNTRYSIDE_YEDIDIA_FOREST:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY);
            space->type=L_TREE;
            space->icon=ICON_COUNTRYSIDE_FOREST;
            space->property = NONE;
            space->space_description = ST_A_VERDANT_FOREST;
            space->room_description = ST_SYLLII;
            space->special = new_special();
            space->special->level_type = LEVEL_YEDIDIA_FOREST;
            space->special->animal_encounter_type = YEDIDIA_ENCOUNTER;
            space->special->animal_encounter_frequency = 50;
            space->special->human_encounter_type = YEDIDIA;
            space->special->human_encounter_frequency = 70;
        break;
        case S_COUNTRYSIDE_YEDIDIA_VILLAGE:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY);
            space->type=L_TREE;
            space->icon=ICON_COUNTRYSIDE_SETTLEMENT;
            space->property = NONE;
            space->space_description = ST_OUTSIDE_A_YEDIDIA_VILLAGE;
            space->room_description = ST_SYLLII;
            space->special = new_special();
            space->special->level_type = LEVEL_YEDIDIA_VILLAGE;
        break;
        case S_COUNTRYSIDE_SHAL_FOREST:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY);
            space->type=L_TREE;
            space->icon=ICON_COUNTRYSIDE_FOREST;
            space->property = NONE;
            space->space_description = ST_A_PEACEFUL_FOREST;
            space->room_description = ST_LISS;
            space->special = new_special();
            space->special->level_type = LEVEL_SHAL_FOREST;
            space->special->animal_encounter_type = SHAL_ENCOUNTER;
            space->special->animal_encounter_frequency = 60;
            space->special->human_encounter_type = SHAL;
            space->special->human_encounter_frequency = 200;
        break;
        case S_COUNTRYSIDE_SHAL_VILLAGE:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY);
            space->type=L_FLOOR;
            space->icon=ICON_COUNTRYSIDE_SETTLEMENT;
            space->property = NONE;
            space->space_description = ST_OUTSIDE_A_PEACEFUL_SHAL_VILLAGE;
            space->room_description = ST_LISS;
            space->special = new_special();
            space->special->level_type = LEVEL_SHAL_VILLAGE;
        break;
        case S_DRY_TREE:
            flag_set(&space->flags, CAN_OCCUPY|TRANSPARENT);
            space->type=L_TREE;
            if (race == NORKRIN)
                space->icon = ICON_PINE_TREE;
            else
                space->icon=ICON_TREE;
            space->property = P_DRY_TREE;
            space->space_description = ST_UNDERNEATH_A_TREE;
        break;
        case S_FINAL_TEST:
            flag_set(&space->flags, CURR_ILLUMINATED|PERM_ILLUMINATED|
              CAN_OCCUPY|AUTOMATIC_EFFECT);
            space->type = L_FINAL_TEST;
            space->icon=ICON_SOMETHING_ELSE;
            space->property = P_FINAL_TEST;
        break;
        case S_FIRE:
            flag_set(&space->flags, CURR_ILLUMINATED|PERM_ILLUMINATED|
              TRANSPARENT|CAN_OCCUPY|AUTOMATIC_EFFECT);
            space->type = L_FIRE;
            space->icon = ICON_FIRE;
            space->property = P_FIRE;
            space->space_description = ST_A_FIRE;
        break;
        case S_FLOOR:
            flag_set(&space->flags, TRANSPARENT|CAN_OCCUPY);
            space->type=L_FLOOR;
            space->icon=ICON_FLOOR;
            space->space_description = ST_THE_GROUND;
        break;
        case S_FOUNTAIN:
            flag_set(&space->flags, TRANSPARENT|CAN_OCCUPY|IS_WATER|SHALLOW|
              AUTOMATIC_EFFECT);
            space->type=L_FOUNTAIN;
            space->icon=ICON_FOUNTAIN;
            space->property=P_SHALLOW_WATER;
            space->space_description = ST_A_FOUNTAIN;
        break;
        case S_FRUIT_TREE:
            flag_set(&space->flags, CAN_OCCUPY|TRANSPARENT);
            space->type=L_TREE;
            if (race == NORKRIN)
                space->icon = ICON_PINE_TREE;
            else
                space->icon=ICON_TREE;
            space->property = P_FRUIT_TREE;
            space->space_description = ST_UNDERNEATH_A_TREE;
        break;
        case S_GREEN_TREE:
            flag_set(&space->flags, CAN_OCCUPY|TRANSPARENT);
            space->type=L_TREE;
            if (race == NORKRIN)
                space->icon = ICON_PINE_TREE;
            else
                space->icon=ICON_TREE;
            space->property = P_GREEN_TREE;
            space->space_description = ST_UNDERNEATH_A_TREE;
        break;
        case S_HEDGE:
            flag_set(&space->flags, AUTOMATIC_EFFECT|CAN_OCCUPY);
            space->type=L_HEDGE;
            space->icon=ICON_HEDGE;
            space->property = P_HEDGE;
            space->space_description = ST_A_HEDGE;
        break;
        case S_HOUSE_DOOR:
            flag_set(&space->flags, AUTOMATIC_EFFECT|DOOR_CLOSED);
            space->type = L_DOOR;
            space->icon = ICON_CLOSED_DOOR;
            space->property = P_HOUSE;
            space->space_description = ST_A_CLOSED_DOOR;
        break;
        case S_INN_ENTRANCE:
            flag_set(&space->flags, AUTOMATIC_EFFECT|EFFECT_BEFORE_MOVE|
              TRANSPARENT|CAN_OCCUPY);
            space->type = L_DOOR;
            space->icon = ICON_OPEN_DOOR;
            space->property = P_INN_ENTRANCE;
            space->space_description = ST_AN_OPEN_DOOR;
        break;
        case S_LOCKED_DOOR:
            flag_set(&space->flags, DOOR_CLOSED|DOOR_LOCKED);
            space->type=L_DOOR;
            space->icon=ICON_CLOSED_DOOR;
            space->space_description = ST_A_CLOSED_DOOR;
        break;
        case S_MAZE_ENTRANCE:
            flag_set(&space->flags, CAN_OCCUPY|TRANSPARENT);
            space->type=L_STAIRWELL_DOWN;
            space->icon=ICON_STAIRWELL_DOWN;
            space->property = P_STAIRWELL_DOWN;
            space->space_description = ST_A_STAIRWELL_LEADING_DOWN;
            space->special = new_special();
            space->special->level_type = LEVEL_MAZE;
        break;
        case S_MECHANICAL:
            flag_set(&space->flags, CAN_OCCUPY|TRANSPARENT);
            space->type = L_MECHANICAL;
            space->icon = ICON_MECHANICAL_SPACE;
            space->space_description = ST_MECHANICAL_EQUIPMENT;
        break;
        case S_COUNTRYSIDE_OCEAN_HIDDEN_ISLE:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY|IS_WATER|SALT_WATER|AUTOMATIC_EFFECT|HAS_SECRET);
            space->type=L_WATER;
            space->icon=ICON_COUNTRYSIDE_OCEAN;
            space->property = P_DEEP_WATER;
            space->space_description = ST_THE_OCEAN;
            space->other = S_COUNTRYSIDE_OCEAN_ISLE;
            space->special = new_special();
            space->special->level_type = LEVEL_OCEAN;
        break;
        case S_COUNTRYSIDE_OCEAN_ISLE:
            flag_set(&space->flags, PERM_ILLUMINATED|CURR_ILLUMINATED|
              CAN_OCCUPY|IS_WATER|SALT_WATER|AUTOMATIC_EFFECT);
            space->type=L_WATER;
            space->icon=ICON_COUNTRYSIDE_FEATURE;
            space->property = P_DEEP_WATER;
            space->space_description = ST_OUTSIDE_A_SMALL_ISLAND;
            space->room_description = ST_THE_OCEAN;
            space->special = new_special();
            space->special->level_type = LEVEL_OCEAN_ISLE;
            flag_set(&space->special->level_flags, LOAD);
        break;
        case S_OPEN_DOOR:
            flag_set(&space->flags, CAN_OCCUPY|TRANSPARENT);
            space->type=L_DOOR;
            space->icon=ICON_OPEN_DOOR;
            space->space_description = ST_AN_OPEN_DOOR;
        break;
        case S_PEAK:
            flag_set(&space->flags, CAN_OCCUPY|TRANSPARENT|EFFECT_BEFORE_MOVE);
            space->type=L_PEAK;
            space->property=P_PEAK;
            space->icon=ICON_PEAK;
            space->space_description = ST_A_ROCKY_PEAK;
        break;
        case S_PEARL_OF_GREAT_PRICE:
            flag_set(&space->flags, TRANSPARENT|CAN_OCCUPY);
            space->type=L_FLOOR;
            space->icon=ICON_FLOOR;
            space->space_description = ST_THE_GROUND;
            this_item = generate_item(current_level, PEARL_OF_GREAT_PRICE);
            this_ilist = (struct ilist *) calloc(1, sizeof(struct ilist));
            this_ilist->me = this_ilist;
            this_ilist->current = this_item;
            this_ilist->next = space->items;
            space->items = this_ilist;
        break;
        case S_PEW:
            flag_set(&space->flags, TRANSPARENT|CAN_OCCUPY);
            space->type=L_FLOOR;
            space->icon=ICON_CHAIR;
            space->space_description = ST_A_PEW;
        break;
        case S_PIT:
            flag_set(&space->flags, CAN_OCCUPY|AUTOMATIC_EFFECT);
            space->type=L_PIT;
            space->icon=ICON_PIT;
            space->property = P_PIT;
            space->space_description = ST_A_PIT;
        break;
        case S_PRAYER_TABLE:
            flag_set(&space->flags, TRANSPARENT|CAN_OCCUPY|AUTOMATIC_EFFECT);
            space->property = P_PRAYER_TABLE;
            space->type=L_FLOOR;
            space->icon=ICON_TABLE_SPACE;
            space->space_description = ST_A_TABLE_OF_PRAYER_CANDLES;
        break;
        case S_RUBBLE:
            flag_set(&space->flags, CAN_OCCUPY|AUTOMATIC_EFFECT|TRANSPARENT);
            space->type=L_FLOOR;
            space->icon=ICON_RUBBLE;
            space->property = P_RUBBLE;
            space->space_description = ST_A_PILE_OF_RUBBLE;
        break;
        case S_SALT_WATER:
            flag_set(&space->flags, CAN_OCCUPY|IS_WATER|TRANSPARENT|SALT_WATER|
              AUTOMATIC_EFFECT);
            space->type=L_WATER;
            space->icon=ICON_WATER;
            space->property = P_DEEP_WATER;
            space->space_description = ST_DEEP_SALTY_WATER;
        break;
        case S_SECRET_PASSAGE:
            flag_set(&space->flags, HAS_SECRET);
            space->type=L_SECRET_PASSAGE;
            space->icon=ICON_STONE_WALL;
            space->space_description = ST_A_STONE_WALL;
            space->other = S_FLOOR; /* What it becomes */
        break;
        case S_SHALLOW_WATER:
            flag_set(&space->flags, CAN_OCCUPY|IS_WATER|TRANSPARENT|
              AUTOMATIC_EFFECT|SHALLOW);
            space->type=L_WATER;
            space->icon=ICON_WATER;
            space->property = P_SHALLOW_WATER;
            space->space_description = ST_A_POOL_OF_WATER;
        break;
        case S_SHELVES:
            flag_set(&space->flags, CAN_OCCUPY|TRANSPARENT);
            space->type = L_FLOOR;
            space->icon=ICON_SHELVES;
            space->space_description = ST_SHELVES;
        break;
        case S_SNOW_ICE:
            flag_set(&space->flags, CAN_OCCUPY|IS_COLD|TRANSPARENT);
            space->type=L_FLOOR;
            space->icon=ICON_SNOW_ICE;
            space->space_description = ST_THE_FROZEN_GROUND;
        break;
        case S_STAINED_GLASS_WINDOW:
            flag_set(&space->flags, TRANSPARENT);
            space->type=L_WINDOW;
            space->icon=ICON_STAINED_GLASS_WINDOW;
            space->space_description = ST_A_STAINED_GLASS_WINDOW;
        break;
        case S_STAIRWELL_DOWN:
            flag_set(&space->flags, CAN_OCCUPY|TRANSPARENT);
            space->type=L_STAIRWELL_DOWN;
            space->icon=ICON_STAIRWELL_DOWN;
            space->property = P_STAIRWELL_DOWN;
            space->space_description = ST_A_STAIRWELL_LEADING_DOWN;
            space->special = new_special();
        break;
        case S_STAIRWELL_UP:
            flag_set(&space->flags, CAN_OCCUPY|TRANSPARENT);
            space->type=L_STAIRWELL_UP;
            space->icon=ICON_STAIRWELL_UP;
            space->property = P_STAIRWELL_UP;
            space->space_description = ST_A_STAIRWELL_LEADING_UP;
            space->special = new_special();
        break;
        case S_STATUE:
            flag_set(&space->flags, TRANSPARENT|CAN_OCCUPY);
            space->type=L_STATUE;
            space->icon=ICON_STATUE_SPACE;
            space->space_description = ST_A_STATUE;
        break;
        case S_TABLE:
            flag_set(&space->flags, TRANSPARENT|CAN_OCCUPY);
            space->type = L_FLOOR;
            space->icon=ICON_TABLE;
            space->space_description = ST_A_TABLE;
        break;
        case S_TOMB:
            space->type = L_WALL;
            space->icon=ICON_STONE_WALL;
            space->space_description = ST_THE_TOMB_OF_A_SAINT;
        break;
        case S_TORCH:
            flag_set(&space->flags, CURR_ILLUMINATED|PERM_ILLUMINATED|
              TRANSPARENT|CAN_OCCUPY);
            space->type = L_FLOOR;
            space->icon=ICON_TORCH;
            space->property = P_TORCH;
            space->space_description = ST_A_TORCH;
        break;
        case S_SAND:
            flag_set(&space->flags, CAN_OCCUPY|AUTOMATIC_EFFECT|TRANSPARENT);
            space->type=L_SAND;
            space->property=P_SAND;
            space->icon=ICON_SAND;
            space->space_description = ST_THE_SANDY_GROUND;
        break;
        case S_WALL:
            space->type=L_WALL;
            space->icon=ICON_STONE_WALL;
            space->space_description = ST_A_STONE_WALL;
        break;
        case S_WATER:
            flag_set(&space->flags, CAN_OCCUPY|IS_WATER|TRANSPARENT|
              AUTOMATIC_EFFECT);
            space->type=L_WATER;
            space->icon=ICON_WATER;
            space->property = P_DEEP_WATER;
            space->space_description = ST_A_POOL_OF_WATER;
        break;
        case S_WINDOW:
            flag_set(&space->flags, TRANSPARENT);
            space->type=L_WINDOW;
            space->icon=ICON_CLEAR_WINDOW;
            space->space_description = ST_A_WINDOW;
        break;
        case S_WOODEN_WALL:
            space->type=L_WALL;
            space->icon=ICON_WOODEN_WALL;
            space->space_description = ST_A_WOODEN_WALL;
        break;
        default:
#ifdef MAC_68k
            pmsg("Error: Out of range space creation identity.");
#else
            perror("Error: Out of range space creation identity.");
#endif
        break;
        }
    }

struct level *dig_level(long level_type, long power_level, long flags, long
  other)
    {
    long i, j, k, l, my_valid, my_total, chosen, so_far, my_continue;
    long exit_border, garbage;
    struct level *result;
    exit_border = TRUE;
    result=(struct level *) calloc(1, sizeof(struct level));
    result->me = result;
    result->flags = flags;
    result->scale = 1; /* Modify this later */
    result->timescale = 1; /* Should only be otherwise for countryside. */
    result->race = NONE;
    result->level_type = level_type;
    result->light_sources = NULL;
    result->entrance_message = ST_EMPTY;
    result->exit_message = ST_EMPTY;
    result->creatures = NULL;
    result->establishment = NULL;
    result->power_level = power_level;
    for(i=0;i<MAX_X;++i)
        for(j=0;j<MAX_Y;++j)
            {
            prepare_space(&result->space[i][j], S_FLOOR, result->race);
            flag_set(&result->space[i][j].flags, AVAILABLE);
            }
    for(i = 0; i < NUM_LOCATION_TYPES; ++i)
        {
        result->location[i].flags = 0;
        result->location[i].identity = locations[i].identity;
        result->location[i].x = 0;
        result->location[i].y = 0;
        }
    if (flag_check(flags, IS_SEASIDE))
        level_add_texture(result, T_SEASIDE, 0);
    for(i = 0; i < MAX_X; ++i)
        {
        flag_reset(&result->space[i][1].flags, AVAILABLE);
        flag_reset(&result->space[i][MAX_Y - 2].flags, AVAILABLE);
        }
    for(j = 0; j < MAX_X; ++j)
        {
        flag_reset(&result->space[1][j].flags, AVAILABLE);
        flag_reset(&result->space[MAX_X - 2][j].flags, AVAILABLE);
        }
    switch (level_type)
        {
        case LEVEL_CAVERN: case LEVEL_SMALL_CAVERN:
            flag_set(&result->flags, PRINT_LEVEL_NUMBER);
            if (level_type == LEVEL_CAVERN)
                flag_set(&result->flags, ALWAYS_FREE);
            result->race = TUZ;
            /* For levels that are mostly stone, we regard stone as AVAILABLE */
            result->environment_name = ST_A_SYSTEM_OF_CAVERNS;
            for(i=0;i<MAX_X;++i)
                for(j=0;j<MAX_Y;++j)
                    {
                    change_space(&result->space[i][j], S_WALL, result->race);
                    flag_reset(&result->space[i][j].flags, AVAILABLE);
                    }
            /* staggering drunk algorithm to create cave */
            i=MAX_X/2;
            j=MAX_Y/2;
            for(;(i>=1 && i<MAX_X-2 && j>=1 && j<MAX_Y-2);)
                {
                change_space(&result->space[i][j], S_FLOOR, result->race);
                flag_set(&result->space[i][j].flags, AVAILABLE);
                switch(trandom()%4)
                    {
                    case 0: ++i; break;
                    case 1: ++j; break;
                    case 2: --i; break;
                    case 3: --j; break;
                    }
                }
            if ((!(level_type == LEVEL_SMALL_CAVERN)) || (trandom() % 3 == 0))
                level_add_texture(result, T_STAIRWELL_DOWN, LEVEL_CAVERN);
            if (power_level > 1)
                level_add_texture(result, T_STAIRWELL_UP, LEVEL_CAVERN);
            else
                level_add_texture(result, T_STAIRWELL_UP, LEVEL_COUNTRYSIDE);
            for(i=0;i<MAX_X;++i)
                for(j=0;j<MAX_Y;++j)
                    {
                    if (flag_check(game_variables->flags, GOING_UP))
                        {
                        if (result->space[i][j].type == L_STAIRWELL_DOWN)
                            {
                flag_reset(&result->space[i][j].flags, AVAILABLE);
                            game_variables->new_level_player_x=i;
                            game_variables->new_level_player_y=j;
                            }
                        }
                    else
                        if (result->space[i][j].type == L_STAIRWELL_UP)
                            {
                flag_reset(&result->space[i][j].flags, AVAILABLE);
                            game_variables->new_level_player_x=i;
                            game_variables->new_level_player_y=j;
                            }
                    }
        /*
         * Commented out because pools had a nasty habit of obliterating
         * stairwells. Even then, I think it was a nice feature.
            while (trandom() % 10 != 0)
                level_add_texture(result, T_POOL, 0);
        */
        break;
        case LEVEL_DUNGEON:
            result->environment_name =
              ST_A_DUNGEON_DEEP_IN_THE_BOWELS_OF_THE_EARTH;
            flag_set(&result->flags, PRINT_LEVEL_NUMBER);
            for(i = 0; i < MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    change_space(&result->space[i][j], S_WALL, result->race);
            while (trandom() % 100 != 0)
                {
                random_select(dungeon_doors, &chosen, &garbage);
                level_add_texture(result, T_DUNGEON_ADD_ROOM, chosen);
                }
            while (level_add_texture(result, T_DUNGEON_JOIN_COMPONENTS, 0))
                my_noop();
            level_add_texture(result, T_STAIRWELL_DOWN, LEVEL_DUNGEON);
            if (power_level > 1)
                level_add_texture(result, T_STAIRWELL_UP, LEVEL_DUNGEON);
            else
                level_add_texture(result, T_STAIRWELL_UP, LEVEL_COUNTRYSIDE);
            for(i=0;i<MAX_X;++i)
                for(j=0;j<MAX_Y;++j)
                    {
                    if (flag_check(game_variables->flags, GOING_UP))
                        {
                        if (result->space[i][j].type == L_STAIRWELL_DOWN)
                            {
                            game_variables->new_level_player_x=i;
                            game_variables->new_level_player_y=j;
                            }
                        }
                    else
                        if (result->space[i][j].type == L_STAIRWELL_UP)
                            {
                            game_variables->new_level_player_x=i;
                            game_variables->new_level_player_y=j;
                            }
                    }
        break;
        case LEVEL_JEC_MIDDLE_PLAINS:
            result->race = JEC;
            flag_set(&result->flags, OUTDOORS);
            result->environment_name = ST_THE_MIDDLE_PLAINS;
            for(i=0;i<MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    {
                    random_select(jec_plains, &k, &l);
                    change_space(&result->space[i][j], k, result->race);
                    flag_set(&result->space[i][j].flags, CAN_HERB);
                    flag_set(&result->space[i][j].flags, AVAILABLE);
                    }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x= 1 + trandom()%(MAX_X / 2);
                game_variables->new_level_player_y= 1 + trandom()%(MAX_Y / 2);
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))));
                }
            break;
        case LEVEL_JEC_TOWN:
            result->race = JEC;
            flag_set(&result->flags, IS_CITY);
            flag_set(&result->flags, OUTDOORS);
            switch(other)
                {
                case JEC_TOWN_HISH:
                    result->environment_name = ST_THE_JEC_TOWN_HISH;
                break;
                case JEC_TOWN_HOS:
                    result->environment_name = ST_THE_JEC_TOWN_HOS;
                break;
                case JEC_TOWN_KANS:
                    result->environment_name = ST_THE_JEC_TOWN_KANS;
                break;
                case JEC_TOWN_LUT:
                    result->environment_name = ST_THE_JEC_TOWN_LUT;
                break;
                case JEC_TOWN_MILN:
                    result->environment_name = ST_THE_JEC_TOWN_MILN;
                break;
                case JEC_TOWN_NUV:
                    result->environment_name = ST_THE_JEC_TOWN_NUV;
                break;
                case JEC_TOWN_TOL:
                    result->environment_name = ST_THE_JEC_TOWN_TOL;
                break;
                case JEC_TOWN_VERN:
                    result->environment_name = ST_THE_JEC_TOWN_VERN;
                break;
                }
            result->entrance_message = ST_YOU_WALK_INTO_A_JEC_TOWN;
            result->exit_message = ST_YOU_LEAVE_A_JEC_TOWN;
            result->scale=60;
            for(i=0;i<100;++i)
                {
                random_select(jec_town_texture, &k, &l);
                level_add_texture(result, k, l);
                }
            my_continue = 1;
            level_add_texture(result, T_ASSIGN_DOOR, CATHEDRAL);
            level_add_texture(result, T_ASSIGN_DOOR, ADVENTURERS_GUILD);
            if (flag_check(flags, IS_SEASIDE))
                level_add_texture(result, T_ASSIGN_DOOR, BOAT_SHOP);
            for(i = 0; i < NUM_DIST_SLOTS; ++i)
                if (jec_doors[i][1] > 0)
                    level_add_texture(result, T_ASSIGN_DOOR, jec_doors[i][0]);
            while (my_continue)
                {
                random_select(jec_doors, &i, &j);
                my_continue = level_add_texture(result, T_ASSIGN_DOOR, i);
                }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x=trandom()%MAX_X;
                game_variables->new_level_player_y=trandom()%MAX_Y;
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  IS_WATER))));
                }
        break;
        case LEVEL_MANSION: case LEVEL_URVANOVESTILLI_MANSION:
            flag_set(&result->flags, PRINT_LEVEL_NUMBER);
            flag_set(&result->flags, ALWAYS_FREE);
            result->environment_name = ST_A_VAST_MANSION;
        if (power_level == 0)
        {
            for(i = 0; i < MAX_X; ++i)
            {
            change_space(&result->space[i][0], S_AUTOMATIC_EXIT, result->race);
            result->space[i][0].special->takes_player_to = countryside;
            result->space[i][0].special->x =
              game_variables->countryside_player_x;
            result->space[i][0].special->y =
              game_variables->countryside_player_y;
            change_space(&result->space[i][MAX_Y - 1], S_AUTOMATIC_EXIT,
              result->race);
            result->space[i][MAX_Y - 1].special->takes_player_to =
              countryside;
            result->space[i][MAX_Y - 1].special->x =
              game_variables->countryside_player_x;
            result->space[i][MAX_Y - 1].special->y =
              game_variables->countryside_player_y;
            }
            for(j = 0; j < MAX_Y; ++j)
            {
            change_space(&result->space[0][j], S_AUTOMATIC_EXIT, result->race);
            result->space[0][j].special->takes_player_to = countryside;
            result->space[0][j].special->x =
              game_variables->countryside_player_x;
            result->space[0][j].special->y =
              game_variables->countryside_player_y;
            change_space(&result->space[MAX_X - 1][j], S_AUTOMATIC_EXIT,
              result->race);
            result->space[MAX_X - 1][j].special->takes_player_to =
              countryside;
            result->space[MAX_X - 1][j].special->x =
              game_variables->countryside_player_x;
            result->space[MAX_X - 1][j].special->y =
              game_variables->countryside_player_y;
            }
        }
            for(i = 1; i < MAX_X - 1; ++i)
                {
                change_space(&result->space[i][1], S_WALL, result->race);
                change_space(&result->space[i][MAX_Y - 2], S_WALL,
                  result->race);
                }
            for(j = 1; j < MAX_Y - 1; ++j)
                {
                change_space(&result->space[1][j], S_WALL, result->race);
                change_space(&result->space[MAX_Y - 2][j], S_WALL,
                  result->race);
                }
            if (power_level == 0)
                {
                change_space(&result->space[MAX_X/2][1], S_OPEN_DOOR,
                  result->race);
                game_variables->new_level_player_x = MAX_X/2;
                game_variables->new_level_player_y = 2;
                flag_reset(&result->space[MAX_X/2][2].flags, AVAILABLE);
                }
            for(i = 0; i < 100; ++i)
                {
                if (trandom() % 2 == 0)
                    level_add_texture(result, T_MANSION_VERTICAL_WALL, 0);
                else
                    level_add_texture(result, T_MANSION_HORIZONTAL_WALL, 0);
                }
            if (power_level > 0 - 7)
                level_add_texture(result, T_STAIRWELL_UP, LEVEL_MANSION);
            if (power_level != 0)
                level_add_texture(result, T_STAIRWELL_DOWN, LEVEL_MANSION);
            level_add_texture(result, T_ASSIGN_MANSION_ROOMS, LEVEL_MANSION);
            while (trandom() % 10 != 0)
                {
                random_select(mansion_doors, &chosen, &garbage);
                level_add_texture(result, T_ASSIGN_DOOR, chosen);
                }
            if (power_level == 0)
                change_space(&result->space[MAX_X/2][1], S_OPEN_DOOR,
                  result->race);
        break;
        case LEVEL_MAZE:
            exit_border = FALSE;
            result->race = URVANOVESTILLI;
            flag_set(&result->flags, PRINT_LEVEL_NUMBER);
            result->environment_name = ST_A_LABYRINTH;
            for(i=0;i<MAX_X;++i)
                for(j=0;j<MAX_Y;++j)
                    if (!((i % 2 == 1) && (j % 2 == 1) && (i < MAX_X - 1) && (j
                      < MAX_Y - 1)))
                        {
                        change_space(&result->space[i][j], S_WALL,
                          result->race);
                        flag_set(&result->space[i][j].flags, AVAILABLE);
                        }
            level_add_texture(result, T_MAZE_JOIN_SPACES, 0);
            level_add_texture(result, T_STAIRWELL_DOWN, LEVEL_MAZE);
            level_add_texture(result, T_STAIRWELL_UP, LEVEL_MAZE);
            /*
            while (trandom() % 20 != 0)
                {
                random_select(maze_texture, &chosen, &garbage);
                level_add_texture(result, chosen, LEVEL_MAZE);
                }
            */
            for(i=0;i<MAX_X;++i)
                for(j=0;j<MAX_Y;++j)
                    {
                    if (result->space[i][j].type==L_FLOOR)
                        result->space[i][j].room_description =
                          ST_A_TWISTY_PASSAGEWAY;
                    }
            for(i=0;i<MAX_X;++i)
                for(j=0;j<MAX_Y;++j)
                    {
                    if (flag_check(game_variables->flags, GOING_UP))
                        {
                        if (result->space[i][j].type == L_STAIRWELL_DOWN)
                            {
                            game_variables->new_level_player_x=i;
                            game_variables->new_level_player_y=j;
                            }
                        }
                    else
                        if (result->space[i][j].type == L_STAIRWELL_UP)
                            {
                            game_variables->new_level_player_x=i;
                            game_variables->new_level_player_y=j;
                            }
                    }
        break;
        case LEVEL_NORKRIN_EAST_RIDGE_MOUNTAINS:
            result->race = NORKRIN;
            flag_set(&result->flags, OUTDOORS);
            result->environment_name = ST_THE_EAST_RIDGE_MOUNTAINS;
            for(i=0;i<MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    {
                    random_select(norkrin_east_ridge_mountains, &k, &l);
                    change_space(&result->space[i][j], k,
                      result->race);
                    flag_set(&result->space[i][j].flags, IS_COLD);
                    flag_set(&result->space[i][j].flags, AVAILABLE);
                    }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x= 1 + trandom()%(MAX_X / 2);
                game_variables->new_level_player_y= 1 + trandom()%(MAX_Y / 2);
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))));
                }
            break;
        case LEVEL_NORKRIN_ICE_PEAKS:
            result->race = NORKRIN;
            flag_set(&result->flags, OUTDOORS);
            result->environment_name = ST_THE_ICE_PEAKS;
            for(i=0;i<MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    {
                    random_select(norkrin_ice_peaks, &k, &l);
                    change_space(&result->space[i][j], k, result->race);
                    flag_set(&result->space[i][j].flags, IS_COLD);
                    flag_set(&result->space[i][j].flags, AVAILABLE);
                    }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x= 1 + trandom()%(MAX_X / 2);
                game_variables->new_level_player_y= 1 + trandom()%(MAX_Y / 2);
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))));
                }
        break;
        case LEVEL_NORKRIN_PLAINS:
            result->race = NORKRIN;
            flag_set(&result->flags, OUTDOORS);
            result->environment_name = ST_THE_NORTHERN_WASTES;
            for(i=0;i<MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    {
                    random_select(norkrin_plains, &k, &l);
                    change_space(&result->space[i][j], k, result->race);
                    flag_set(&result->space[i][j].flags, AVAILABLE);
                    }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x= 1 + trandom()%(MAX_X / 2);
                game_variables->new_level_player_y= 1 + trandom()%(MAX_Y / 2);
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))));
                }
            break;
        case LEVEL_OCEAN:
            flag_set(&result->flags, OUTDOORS);
            result->environment_name = ST_THE_OCEAN;
            for(i=0;i<MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    {
                    random_select(ocean, &k, &l);
                    change_space(&result->space[i][j], k, result->race);
                    flag_set(&result->space[i][j].flags, AVAILABLE);
                    }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x= 1 + trandom()%(MAX_X / 2);
                game_variables->new_level_player_y= 1 + trandom()%(MAX_Y / 2);
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))));
                }
            break;
        case LEVEL_SHAL_FOREST:
            result->race = SHAL;
            flag_set(&result->flags, OUTDOORS);
            result->environment_name = ST_A_PEACEFUL_FOREST;
            for(i=0;i<MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    {
                    random_select(shal_forest, &k, &l);
                    change_space(&result->space[i][j], k, result->race);
                    flag_set(&result->space[i][j].flags, CAN_HERB);
                    flag_set(&result->space[i][j].flags, AVAILABLE);
                    }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x= 1 + trandom()%(MAX_X / 2);
                game_variables->new_level_player_y= 1 + trandom()%(MAX_Y / 2);
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))));
                }
            break;
        case LEVEL_SHAL_VILLAGE:
            flag_set(&result->flags, IS_CITY);
            result->race = SHAL;
            flag_set(&result->flags, OUTDOORS);
            switch(other)
                {
                case SHAL_VILLAGE_HESRI:
                    result->environment_name = ST_THE_SHAL_VILLAGE_HESRI;
                break;
                case SHAL_VILLAGE_INGSA:
                    result->environment_name = ST_THE_SHAL_VILLAGE_INGSA;
                break;
                case SHAL_VILLAGE_KASSILLA:
                    result->environment_name = ST_THE_SHAL_VILLAGE_KASSILLA;
                break;
                case SHAL_VILLAGE_LITRI:
                    result->environment_name = ST_THE_SHAL_VILLAGE_LITRI;
                break;
                case SHAL_VILLAGE_MAC_68kEP:
                    result->environment_name = ST_THE_SHAL_VILLAGE_MAC_68kEP;
                break;
                case SHAL_VILLAGE_NOLDI:
                    result->environment_name = ST_THE_SHAL_VILLAGE_NOLDI;
                break;
                case SHAL_VILLAGE_SALN:
                    result->environment_name = ST_THE_SHAL_VILLAGE_SALN;
                break;
                case SHAL_VILLAGE_SIF:
                    result->environment_name = ST_THE_SHAL_VILLAGE_SIF;
                break;
                }
            result->entrance_message = ST_YOU_WALK_INTO_A_SHAL_VILLAGE;
            result->exit_message = ST_YOU_LEAVE_A_SHAL_VILLAGE;
            result->scale=60;
            for(i=0;i<100;++i)
                {
                random_select(jec_town_texture, &k, &l);
                level_add_texture(result, k, l);
                }
            my_continue = 1;
            level_add_texture(result, T_ASSIGN_DOOR, CATHEDRAL);
            level_add_texture(result, T_ASSIGN_DOOR, ADVENTURERS_GUILD);
            if (flag_check(flags, IS_SEASIDE))
                level_add_texture(result, T_ASSIGN_DOOR, BOAT_SHOP);
            for(i = 0; i < NUM_DIST_SLOTS; ++i)
                if (shal_doors[i][1] > 0)
                    level_add_texture(result, T_ASSIGN_DOOR, shal_doors[i][0]);
            while (my_continue)
                {
                random_select(shal_doors, &i, &j);
                my_continue = level_add_texture(result, T_ASSIGN_DOOR, i);
                }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x= 1 + trandom()%(MAX_X / 2);
                game_variables->new_level_player_y= 1 + trandom()%(MAX_Y / 2);
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  IS_WATER))));
                }
        break;
        case LEVEL_TEST:
            result->environment_name = ST_TEST;
            for(i=0;i<MAX_X;++i)
                for(j=0;j<MAX_Y;++j)
                    result->space[i][j].room_description = ST_TEST;
        break;
        case LEVEL_TUZ_EAST_RIDGE_MOUNTAINS:
            flag_set(&result->flags, OUTDOORS);
            result->environment_name = ST_THE_EAST_RIDGE_MOUNTAINS;
            for(i=0;i<MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    {
                    random_select(tuz_east_ridge_mountains, &k, &l);
                    change_space(&result->space[i][j], k, result->race);
                    flag_set(&result->space[i][j].flags, CAN_HERB);
                    flag_set(&result->space[i][j].flags, AVAILABLE);
                    }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x= 1 + trandom()%(MAX_X / 2);
                game_variables->new_level_player_y= 1 + trandom()%(MAX_Y / 2);
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))));
                }
            break;
        case LEVEL_TUZ_FOREST:
            result->race = TUZ;
            flag_set(&result->flags, OUTDOORS);
            result->environment_name = ST_A_DENSE_FOREST;
            for(i=0;i<MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    {
                    random_select(tuz_forest, &k, &l);
                    change_space(&result->space[i][j], k, result->race);
                    flag_set(&result->space[i][j].flags, CAN_HERB);
                    flag_set(&result->space[i][j].flags, AVAILABLE);
                    }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x= 1 + trandom()%(MAX_X / 2);
                game_variables->new_level_player_y= 1 + trandom()%(MAX_Y / 2);
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))));
                }
            break;
        case LEVEL_TUZ_VILLAGE:
            flag_set(&result->flags, IS_CITY);
            result->race = TUZ;
            flag_set(&result->flags, OUTDOORS);
            switch(other)
                {
                case TUZ_VILLAGE_BORD:
                    result->environment_name = ST_THE_TUZ_VILLAGE_BORD;
                break;
                case TUZ_VILLAGE_DOZ:
                    result->environment_name = ST_THE_TUZ_VILLAGE_DOZ;
                break;
                case TUZ_VILLAGE_GHIRTON:
                    result->environment_name = ST_THE_TUZ_VILLAGE_GHIRTON;
                break;
                case TUZ_VILLAGE_GORF:
                    result->environment_name = ST_THE_TUZ_VILLAGE_GORF;
                break;
                case TUZ_VILLAGE_KANDRHI:
                    result->environment_name = ST_THE_TUZ_VILLAGE_KANDRHI;
                break;
                case TUZ_VILLAGE_KORG:
                    result->environment_name = ST_THE_TUZ_VILLAGE_KORG;
                break;
                case TUZ_VILLAGE_KUNDER:
                    result->environment_name = ST_THE_TUZ_VILLAGE_KUNDER;
                break;
                case TUZ_VILLAGE_LUZZ:
                    result->environment_name = ST_THE_TUZ_VILLAGE_LUZZ;
                break;
                case TUZ_VILLAGE_RASTI:
                    result->environment_name = ST_THE_TUZ_VILLAGE_RASTI;
                break;
                case TUZ_VILLAGE_RHALLTIR:
                    result->environment_name = ST_THE_TUZ_VILLAGE_RHALLTIR;
                break;
                case TUZ_VILLAGE_RHAV:
                    result->environment_name = ST_THE_TUZ_VILLAGE_RHAV;
                break;
                case TUZ_VILLAGE_STORNGE:
                    result->environment_name = ST_THE_TUZ_VILLAGE_STORNGE;
                break;
                case TUZ_VILLAGE_TUZZNIR:
                    result->environment_name = ST_THE_TUZ_VILLAGE_TUZZNIR;
                break;
                case TUZ_VILLAGE_WAZ:
                    result->environment_name = ST_THE_TUZ_VILLAGE_WAZ;
                break;
                case TUZ_VILLAGE_ZOLK:
                    result->environment_name = ST_THE_TUZ_VILLAGE_ZOLK;
                break;
                case TUZ_VILLAGE_ZUT:
                    result->environment_name = ST_THE_TUZ_VILLAGE_ZUT;
                break;
                }
            result->entrance_message = ST_YOU_WALK_INTO_A_TUZ_VILLAGE;
            result->exit_message = ST_YOU_LEAVE_A_TUZ_VILLAGE;
            result->scale=60;
            for(i=0;i<100;++i)
                {
                random_select(tuz_village_texture, &k, &l);
                level_add_texture(result, k, l);
                }
            my_continue = 1;
            level_add_texture(result, T_ASSIGN_DOOR, CATHEDRAL);
            level_add_texture(result, T_ASSIGN_DOOR, ADVENTURERS_GUILD);
            for(i = 0; i < NUM_DIST_SLOTS; ++i)
                if (tuz_doors[i][1] > 0)
                    level_add_texture(result, T_ASSIGN_DOOR, tuz_doors[i][0]);
            if (flag_check(flags, IS_SEASIDE))
                level_add_texture(result, T_ASSIGN_DOOR, BOAT_SHOP);
            while (my_continue)
                {
                random_select(tuz_doors, &i, &j);
                my_continue = level_add_texture(result, T_ASSIGN_DOOR, i);
                }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x= 1 + trandom()%(MAX_X / 2);
                game_variables->new_level_player_y= 1 + trandom()%(MAX_Y / 2);
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  IS_WATER))));
                }
        break;
        case LEVEL_URVANOVESTILLI_CITY:
            flag_set(&result->flags, IS_CITY);
            result->race = URVANOVESTILLI;
            flag_set(&result->flags, OUTDOORS);
            switch(other)
                {
                case URVANOVESTILLI_CITY_CANSOMILTRAMA:
                    result->environment_name =
                      ST_THE_URVANOVESTILLI_CITY_CANSOMILTRAMA;
                break;
                case URVANOVESTILLI_CITY_CAPITELLO:
                    result->environment_name =
                      ST_THE_URVANOVESTILLI_CITY_CAPITELLO;
                break;
                case URVANOVESTILLI_CITY_ELHANNADAN:
                    result->environment_name =
                      ST_THE_URVANOVESTILLI_CITY_ELHANNADAN;
                break;
                case URVANOVESTILLI_CITY_ELIRE:
                    result->environment_name =
                      ST_THE_URVANOVESTILLI_CITY_ELIRE;
                break;
                case URVANOVESTILLI_CITY_ELMIRTRA:
                    result->environment_name =
                      ST_THE_URVANOVESTILLI_CITY_ELMIRTRA;
                break;
                case URVANOVESTILLI_CITY_KILNATRONA:
                    result->environment_name =
                      ST_THE_URVANOVESTILLI_CITY_KILNATRONA;
                break;
                case URVANOVESTILLI_CITY_MISTRELLI:
                    result->environment_name =
                      ST_THE_URVANOVESTILLI_CITY_MISTRELLI;
                break;
                case URVANOVESTILLI_CITY_SANDRELLA:
                    result->environment_name =
                      ST_THE_URVANOVESTILLI_CITY_SANDRELLA;
                break;
                case URVANOVESTILLI_CITY_STANDRILNATRELLA:
                    result->environment_name =
                      ST_THE_URVANOVESTILLI_CITY_STANDRILNATRELLA;
                break;
                case URVANOVESTILLI_CITY_TALMIRNATRII:
                    result->environment_name =
                      ST_THE_URVANOVESTILLI_CITY_TALMIRNATRII;
                break;
                }
            result->entrance_message = ST_YOU_WALK_INTO_AN_URVANOVESTILLI_CITY;
            result->exit_message = ST_YOU_LEAVE_AN_URVANOVESTILLI_CITY;
            result->scale=60;
            my_total = 0;
            for(i=0;i<NUM_DIST_SLOTS;++i)
                my_total += urvanovestilli_city_texture[i][1];
            for(i=0;i<1000;++i)
                {
                chosen = trandom() % my_total + 1;
                so_far = 0;
                for(j=0;j<NUM_DIST_SLOTS;++j)
                    {
                    so_far += urvanovestilli_city_texture[j][1];
                    if ((so_far - urvanovestilli_city_texture[j][1] < chosen)
                      && (chosen <= so_far))
                        level_add_texture(result,
                          urvanovestilli_city_texture[j][0],
                          urvanovestilli_city_texture[j][2]);
                    }
                }
            switch(other)
                {
                case URVANOVESTILLI_CITY_MISTRELLI:
                    for(i = 0; i < 100; ++i)
                        level_add_texture(result, T_HIDDEN_MAZE_ENTRANCE, 0);
                    for(i = 0; i < MAX_X; ++i)
                        for(j = 0; j < MAX_Y; ++j)
                            if (result->space[i][j].type == L_STATUE)
                                if (trandom() % 2 == 0)
                                    {
                                    flag_set(&result->space[i][j].flags,
                                      HAS_SECRET);
                                    result->space[i][j].other =
                                      S_MAZE_ENTRANCE;
                                    }
                break;
                case URVANOVESTILLI_CITY_STANDRILNATRELLA:
                    level_add_texture(result, T_ASSIGN_DOOR, MENTOR);
                break;
                }
            level_add_texture(result, T_ASSIGN_DOOR, CATHEDRAL);
            level_add_texture(result, T_ASSIGN_DOOR, TINKERERS_GUILD);
            level_add_texture(result, T_ASSIGN_DOOR, ADVENTURERS_GUILD);
            if (flag_check(flags, IS_SEASIDE))
                level_add_texture(result, T_ASSIGN_DOOR, BOAT_SHOP);
            for(i = 0; i < NUM_DIST_SLOTS; ++i)
                if (urvanovestilli_doors[i][1] > 0)
                    level_add_texture(result, T_ASSIGN_DOOR, urvanovestilli_doors[i][0]);
            my_continue = 1;
            while (my_continue)
                {
                random_select(urvanovestilli_doors, &i, &j);
                my_continue = level_add_texture(result, T_ASSIGN_DOOR, i);
                }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x= 1 + trandom()%(MAX_X / 2);
                game_variables->new_level_player_y= 1 + trandom()%(MAX_Y / 2);
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  IS_WATER))));
                }
        break;
        case LEVEL_URVANOVESTILLI_EAST_RIDGE_MOUNTAINS:
            result->race = URVANOVESTILLI;
            flag_set(&result->flags, OUTDOORS);
            result->environment_name = ST_THE_EAST_RIDGE_MOUNTAINS;
            for(i=0;i<MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    {
                    random_select(urvanovestilli_east_ridge_mountains, &k, &l);
                    change_space(&result->space[i][j], k, result->race);
                    flag_set(&result->space[i][j].flags, CAN_HERB);
                    flag_set(&result->space[i][j].flags, AVAILABLE);
                    }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x= 1 + trandom()%(MAX_X / 2);
                game_variables->new_level_player_y= 1 + trandom()%(MAX_Y / 2);
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))));
                }
            break;
        case LEVEL_URVANOVESTILLI_FOG_VALLEY:
            result->race = URVANOVESTILLI;
            flag_set(&result->flags, OUTDOORS);
            result->environment_name = ST_THE_FOG_VALLEY;
            for(i=0;i<MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    {
                    random_select(urvanovestilli_fog_valley, &k, &l);
                    change_space(&result->space[i][j], k, result->race);
                    flag_set(&result->space[i][j].flags, CAN_HERB);
                    flag_set(&result->space[i][j].flags, AVAILABLE);
                    }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x= 1 + trandom()%(MAX_X / 2);
                game_variables->new_level_player_y= 1 + trandom()%(MAX_Y / 2);
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))));
                }
            break;
        case LEVEL_URVANOVESTILLI_PLAINS:
            result->race = URVANOVESTILLI;
            flag_set(&result->flags, OUTDOORS);
            result->environment_name = ST_THE_MIDDLE_PLAINS;
            for(i=0;i<MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    {
                    random_select(urvanovestilli_plains, &k, &l);
                    change_space(&result->space[i][j], k, result->race);
                    flag_set(&result->space[i][j].flags, CAN_HERB);
                    flag_set(&result->space[i][j].flags, AVAILABLE);
                    }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x= 1 + trandom()%(MAX_X / 2);
                game_variables->new_level_player_y= 1 + trandom()%(MAX_Y / 2);
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))));
                }
            break;
        case LEVEL_YEDIDIA_FOREST:
            result->race = YEDIDIA;
            flag_set(&result->flags, OUTDOORS);
            result->environment_name = ST_A_VERDANT_FOREST;
            for(i=0;i<MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    {
                    random_select(yedidia_forest, &k, &l);
                    change_space(&result->space[i][j], k, result->race);
                    flag_set(&result->space[i][j].flags, CAN_HERB);
                    flag_set(&result->space[i][j].flags, AVAILABLE);
                    }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x= 1 + trandom()%(MAX_X / 2);
                game_variables->new_level_player_y= 1 + trandom()%(MAX_Y / 2);
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))));
                }
            break;
        case LEVEL_YEDIDIA_VILLAGE:
            flag_set(&result->flags, IS_CITY);
            flag_set(&result->flags, CAN_HERB);
            result->race = YEDIDIA;
            flag_set(&result->flags, OUTDOORS);
            switch(other)
                {
                case YEDIDIA_VILLAGE_DOSSE:
                    result->environment_name = ST_THE_YEDIDIA_VILLAGE_DOSSE;
                break;
                case YEDIDIA_VILLAGE_EHOLINA:
                    result->environment_name = ST_THE_YEDIDIA_VILLAGE_EHOLINA;
                break;
                case YEDIDIA_VILLAGE_FILTRI:
                    result->environment_name = ST_THE_YEDIDIA_VILLAGE_FILTRI;
                break;
                case YEDIDIA_VILLAGE_FISRATIL:
                    result->environment_name = ST_THE_YEDIDIA_VILLAGE_FISRATIL;
                break;
                case YEDIDIA_VILLAGE_FISSK:
                    result->environment_name = ST_THE_YEDIDIA_VILLAGE_FISSK;
                break;
                case YEDIDIA_VILLAGE_SALINA:
                    result->environment_name = ST_THE_YEDIDIA_VILLAGE_SALINA;
                break;
                case YEDIDIA_VILLAGE_TIRNA:
                    result->environment_name = ST_THE_YEDIDIA_VILLAGE_TIRNA;
                break;
                case YEDIDIA_VILLAGE_VASLI:
                    result->environment_name = ST_THE_YEDIDIA_VILLAGE_VASLI;
                break;
                }
            result->entrance_message = ST_YOU_WALK_INTO_A_YEDIDIA_VILLAGE;
            result->exit_message = ST_YOU_LEAVE_A_YEDIDIA_VILLAGE;
            result->scale=20;
            my_total = 0;
            if (trandom()%2 == 0)
                level_add_texture(result, T_STREAM, 0);
            for(i=0;i<NUM_DIST_SLOTS;++i)
                my_total += yedidia_city_texture[i][1];
            for(i=0;i<100;++i)
                {
                chosen = trandom() % my_total + 1;
                so_far = 0;
                for(j=0;j<NUM_DIST_SLOTS;++j)
                    {
                    so_far += yedidia_city_texture[j][1];
                    if ((so_far - yedidia_city_texture[j][1] < chosen)
                      && (chosen <= so_far))
                        level_add_texture(result, yedidia_city_texture[j][0],
                          yedidia_city_texture[j][2]);
                    }
                }
            my_continue = 1;
            level_add_texture(result, T_ASSIGN_DOOR, CATHEDRAL);
            level_add_texture(result, T_ASSIGN_DOOR, ADVENTURERS_GUILD);
            if (flag_check(flags, IS_SEASIDE))
                level_add_texture(result, T_ASSIGN_DOOR, BOAT_SHOP);
            for(i = 0; i < NUM_DIST_SLOTS; ++i)
                if (yedidia_doors[i][1] > 0)
                    level_add_texture(result, T_ASSIGN_DOOR, yedidia_doors[i][0]);
            while (my_continue)
                {
                random_select(yedidia_doors, &i, &j);
                my_continue = level_add_texture(result, T_ASSIGN_DOOR, i);
                }
            for(my_valid=0;(!my_valid);)
                {
                game_variables->new_level_player_x= 1 + trandom()%(MAX_X / 2);
                game_variables->new_level_player_y= 1 + trandom()%(MAX_Y / 2);
                my_valid=(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  CAN_OCCUPY) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  OCCUPIED))) && (!(flag_check(result->space[game_variables->
                  new_level_player_x][game_variables->new_level_player_y].flags,
                  IS_WATER))));
                }
        break;
        default:
            pmsg("You don't look like Peter Pan, but you've managed to escape to never-never land!");
            pmsg("Hit '>' to return to the countryside...");
            morewait();
            game_variables->new_level_player_x = MAX_X / 2;
            game_variables->new_level_player_y = MAX_Y / 2;
            result->environment_name = ST_NEVER_NEVER_LAND;
            for(i = 0; i < MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    {
                    result->space[i][j].property = P_EXIT;
                    result->space[i][j].special = new_special();
                    result->space[i][j].special->takes_player_to = countryside;
                    result->space[i][j].special->x = MAX_X/2;
                    result->space[i][j].special->y = MAX_Y/2;
                    }
        break;
        }
    if (exit_border)
        {
        for(i=0;i<MAX_X;++i)
            {
            result->space[i][0].property = P_EXIT;
            flag_set(&result->space[i][0].flags, AUTOMATIC_EFFECT);
            result->space[i][0].special = new_special();
            result->space[i][MAX_Y - 1].property = P_EXIT;
            flag_set(&result->space[i][MAX_Y - 1].flags, AUTOMATIC_EFFECT);
            result->space[i][MAX_Y - 1].special = new_special();
            }
        for(j=0;j<MAX_Y;++j)
            {
            result->space[0][j].property = P_EXIT;
            flag_set(&result->space[0][j].flags, AUTOMATIC_EFFECT);
            result->space[0][j].special = new_special();
            result->space[MAX_X - 1][j].property = P_EXIT;
            flag_set(&result->space[MAX_X - 1][j].flags, AUTOMATIC_EFFECT);
            result->space[MAX_X - 1][j].special = new_special();
            }
        }
    if (flag_check(flags, L_CLOAKED_BY_MIST))
    for(i=0;i<MAX_X;++i)
        for(j=0;j<MAX_Y;++j)
            flag_set(&result->space[i][j].flags, S_CLOAKED_BY_MIST);
    /* Possibly move this elsewhere -- keeping it here for now, because */
    /* it puts the player in the right spot. */
    return result;
    }
