/***************************************************************************/
/* The Minstrel's Song is (c) 1998-2005 by Jonathan Hayward.  See the file */
/* lib/copyrigh.txt for a copyright and license statement.                 */
/***************************************************************************/

#include "tms.h"
#include <math.h>

#ifdef CODE_SEGMENT
#pragma segment LEVEL3_C_A
#endif

/* Attempt to add a piece of texture to a random location; return value */
/* indicates whether it was successful.  */

long level_add_texture(struct level *this_level, long my_texture_type, long
  other)
    {
    long i, j, k, l, m, n, o, p, numdoors, curr_doors, my_selected, my_success,
        my_continue, num_spaces, curr_spaces, my_valid, found_space, best_x,
        best_y, assigned_space, assigned_first_door, chosen;
    my_success = 0;
    i = j = k = l = o = p = 0;
    unmark_level(this_level);
    switch(my_texture_type)
        {
        case T_ASSIGN_DOOR: /* Assign a special function to a door */
            numdoors = curr_doors =0;
            for(i=0;i<MAX_X;++i)
                for(j=0;j<MAX_Y;++j)
                    if (this_level->space[i][j].type == L_DOOR)
                        if (this_level->space[i][j].property==NONE)
                            ++numdoors;
            if (numdoors > 0)
                {
                my_selected=trandom()%numdoors;
                for(i=0;i<MAX_X;++i)
                    for(j=0;j<MAX_Y;++j)
                        if ((this_level->space[i][j].property == NONE) &&
                        (this_level->space[i][j].type == L_DOOR))
                            if (this_level->space[i][j].special==NULL)
                                {
                                if (curr_doors==my_selected)
                                    {
                                    assign_door(this_level, i, j, other);
                                    my_success=1;
                                    }
                                ++curr_doors;
                                }
                my_success=1;
                }
        break; /* T_ASSIGN_DOOR */
        case T_ASSIGN_MANSION_ROOMS:
            for(i = 1; i < MAX_X - 1; ++i)
                for(j = 1; j < MAX_Y - 1; ++j)
                    if ((this_level->space[i][j].type == L_FLOOR) &&
                      (this_level->space[i][j].room_description == ST_EMPTY))
                        {
                        k = i;
                        while ((this_level->space[k + 1][j].type == L_FLOOR) &&
                          (this_level->space[k + 1][j].room_description ==
                          ST_EMPTY))
                            ++k;
                        l = j;
                        while (l < MAX_Y && (this_level->space[i][l + 1].type
                          == L_FLOOR) &&
                          (this_level->space[i][l + 1].room_description ==
                          ST_EMPTY))
                            ++l;
                        switch(other)
                            {
                            case LEVEL_MANSION:
                                random_select(mansion_rooms, &m, &n);
                            break;
                            default:
                                m = ROOM_BARE_ROOM;
                            break;
                            }
                        for(o = i; o <= k; ++o)
                            for(p = j; p <= l; ++p)
                                prepare_room_space(this_level,
                                  &this_level->space[o][p], m);
                        }
        break;
        case T_BUILDING_CIRCULAR:
            k=2;
            while
                (trandom()%2)
                ++k;
            l=k;
            m=k + 1 + trandom()%(MAX_X-2*k-3);
            n=l + 1 + trandom()%(MAX_Y-2*l-3);
            mark_circle(this_level, m, n, k + 1);
            my_valid=1;
            for(o=m - k - 1;o<m+k+2;++o)
                for(p=n - l - 1;p<n+l+2;++p)
                    if ((0 <= o) && (o < MAX_X) && (0 <= p) && (p < MAX_Y))
                        if (flag_check(this_level->space[o][p].flags, MARKED))
                            my_valid=(my_valid &&
                              flag_check(this_level->space[o][p].flags,
                              AVAILABLE));
            unmark_level(this_level);
            if (my_valid)
                {
                mark_circle(this_level, m, n, k);
                i = m;
                j = n;
                find_boundary(this_level, &i, &j);
                change_space(&this_level->space[i][j], S_OPEN_DOOR,
                  this_level->race);
                for(o=m-k;o<m+k+1;++o)
                    for(p=n-l;p<n+l+1;++p)
                        if ((0 <= o) && (o <= MAX_X) && (0 <= p) && (p <=
                          MAX_Y))
                            if (flag_check(this_level->space[o][p].flags,
                              MARKED))
                                {
                                change_space(&this_level->space[o][p], other,
                                  this_level->race);
                                /* Keep people from walking into buildings,
                                  even buildings woven out of living trees */
                                flag_reset(&this_level->space[o][p].flags,
                                  CAN_OCCUPY);
                                for(i=o-1;i<o+2;++i)
                                    for(j=p-1;j<p+2;++j)
                                        flag_reset(
                                          &this_level->space[o][p].flags,
                                          AVAILABLE);
                                }
                my_success=1;
                }
        break; /* T_BUILDING_CIRCULAR */
        case T_BUILDING_POOL:
            k=2;
            while
                (trandom()%2)
                ++k;
            l=5*k*k;
            m=k + 1 + trandom()%(MAX_X-2*k-3);
            n=k + 1 + trandom()%(MAX_Y-2*k-3);
            my_valid=1;
            mark_pool(this_level, m, n, k, l);
            for(o=0;o<MAX_X;++o)
                for(p=0;p<MAX_Y;++p)
                    if ((0 <= o) && (o <= MAX_X) && (0 <= p) && (p <= MAX_Y))
                        if (flag_check(this_level->space[o][p].flags, MARKED))
                            {
                            my_valid=(my_valid &&
                              flag_check(this_level->space[o][p].flags,
                              AVAILABLE));
                            if ((0 < o) && (o < MAX_X) && (0 < p) && (p <
                              MAX_Y))
                                for(i=o-1; i<o+2;++i)
                                    for(j=p-1; j < p - 2; ++j)
                                        my_valid=(my_valid && flag_check(
                                          this_level->space[i][j].flags,
                                          AVAILABLE));
                            }
            if (my_valid)
                {
                i = m;
                j = n;
                find_boundary(this_level, &i, &j);
                change_space(&this_level->space[i][j], S_OPEN_DOOR,
                  this_level->race);
                if (other == S_GREEN_TREE) /* Yedidia building */
                    {
                    this_level->space[i][j].space_description =
                        ST_A_DOORWAY_OF_HANGING_VINES;
                    this_level->space[i][j].room_description =
                        ST_A_BUILDING_WOVEN_OUT_OF_LIVING_TREES;
                    }
                else
                    this_level->space[i][j].room_description = ST_A_BUILDING;
                for(o=0;o<MAX_X;++o)
                    for(p=0;p<MAX_Y;++p)
                        if (flag_check(this_level->space[o][p].flags, MARKED))
                            {
                            if (other == S_GREEN_TREE) /* Yedidia building */
                            this_level->space[i][j].room_description =
                                ST_A_BUILDING_WOVEN_OUT_OF_LIVING_TREES;
                            else
                                this_level->space[i][j].room_description =
                                  ST_A_BUILDING;
                            change_space(&this_level->space[o][p], other,
                              this_level->race);
                            flag_reset(&this_level->space[o][p].flags,
                              TRANSPARENT);
                            flag_reset(&this_level->space[o][p].flags,
                              CAN_OCCUPY);
                            for(i=o-1;i<o+2;++i)
                                for(j=p-1;j<p+2;++j)
                                    if ((0 <= i) && (i <= MAX_X) && (0 <= j) &&
                                      (j <= MAX_Y))
                                        flag_reset(
                                          &this_level->space[o][p].flags,
                                          AVAILABLE);
                            }
                my_success=1;
                }
                unmark_level(this_level);
        break; /* T_BUILDING_POOL */
        case T_BUILDING_RECTANGULAR:
            k=2;
            while
                (trandom()%2)
                ++k;
            l=2;
            while
                (trandom()%2)
                ++l;
            m=trandom()%(MAX_X-k-2);
            n=trandom()%(MAX_Y-l-2);
            my_valid=1;
            for(o=m;o<m+k+3;++o)
                for(p=n;p<n+l+3;++p)
                    my_valid=(my_valid &&
                    flag_check(this_level->space[o][p].flags, AVAILABLE));
            if (my_valid)
                {
                for(o=m+1;o<m+k+2;++o)
                    for(p=n+1;p<n+l+2;++p)
                        {
                        change_space(&this_level->space[o][p], other,
                          this_level->race);
                        flag_reset(&this_level->space[o][p].flags, CAN_OCCUPY);
                        }
                for(o=m;o<m+k+2;++o)
                    for(p=n;p<n+l+2;++p)
                        flag_reset(&this_level->space[o][p].flags, AVAILABLE);
                switch(trandom()%4)
                    {
                    case 0:
                        change_space(&this_level->space[m+trandom()%k+1][n+1],
                          S_OPEN_DOOR, this_level->race);
                    break;
                    case 1:
                        change_space(&this_level->space[m+trandom()%k+1][n+l+1],
                          S_OPEN_DOOR, this_level->race);
                    break;
                    case 2:
                        change_space(&this_level->space[m+1][trandom()%l+n+1],
                          S_OPEN_DOOR, this_level->race);
                    break;
                    case 3:
                        change_space(&this_level->space[m+k+1]
                          [trandom()%l+n+1], S_OPEN_DOOR, this_level->race);
                    break;
                    }
                my_success = 1;
                }
        break; /* T_BUILDING_RECTANGULAR */
        case T_BUILDING_ROUNDED: /* corners rounded off an otherwise */
                                 /* rectangular room */
            k=5;
            while
                (trandom()%2)
                ++k;
            l=5;
            while
                (trandom()%2)
                ++l;
            m=trandom()%(MAX_X-k-2);
            n=trandom()%(MAX_Y-l-2);
            my_valid=1;
            for(o=m;o<m+k+2;++o)
                for(p=n;p<n+l+2;++p)
                    if (!(((o == m + 1) || (o == m + k)) && ((p == n + 1) ||
                      (p == n + l))))
                        my_valid=(my_valid &&
                          flag_check(this_level->space[o][p].flags, AVAILABLE));
            if (my_valid)
                {
                for(o=m+1;o<m+k+2;++o)
                    for(p=n+1;p<n+l+2;++p)
                        if (!(((o == m + 1) || (o == m + k + 1)) && ((p == n +
                          1) || (p == n + l + 1))))
                            {
                            change_space(&this_level->space[o][p], other,
                              this_level->race);
                            flag_reset(&this_level->space[o][p].flags,
                              CAN_OCCUPY);
                            }
                for(o=m;o<m+k+3;++o)
                    for(p=n;p<n+l+3;++p)
                        if (!(((o == m) || (o == m + k + 1)) && ((p == n) || (p
                          == n + l + 1))))
                            flag_reset(&this_level->space[o][p].flags,
                              AVAILABLE);
                switch(trandom()%4)
                    {
                    case 0:
                        change_space(&this_level->space[m+trandom()%(k - 4) + 2]
                          [n + 1], S_OPEN_DOOR, this_level->race);
                    break;
                    case 1:
                        change_space(&this_level->space[m+trandom()%(k - 4) + 2]
                          [n+l + 1], S_OPEN_DOOR, this_level->race);
                    break;
                    case 2:
                        change_space(&this_level->space[m + 1]
                          [trandom()%(l - 4)+n + 2], S_OPEN_DOOR,
                          this_level->race);
                    break;
                    case 3:
                        change_space(&this_level->space[m+k + 1]
                          [trandom()%(l - 4)+n + 2], S_OPEN_DOOR,
                          this_level->race);
                    break;
                    }
                my_success = 1;
                }
        break; /* T_BUILDING_ROUNDED */
        case T_BUILDING_SQUARE: /* corners rounded off an otherwise */
                                 /* rectangular room */
            k=3;
            while
                (trandom()%2)
                ++k;
            l = k;
            m=trandom()%(MAX_X-k-2);
            n=trandom()%(MAX_Y-l-2);
            my_valid=1;
            for(o=m;o<m+k+3;++o)
                for(p=n;p<n+l+3;++p)
                        my_valid=(my_valid &&
                        flag_check(this_level->space[o][p].flags, AVAILABLE));
            if (my_valid)
                {
                for(o=m + 1;o<m+k+2;++o)
                    for(p=n + 1;p<n+l+2;++p)
                        {
                        change_space(&this_level->space[o][p], other,
                          this_level->race);
                        flag_reset(&this_level->space[o][p].flags, CAN_OCCUPY);
                        }
                for(o=m;o<m+k+2;++o)
                    for(p=n;p<n+l+2;++p)
                        flag_reset(&this_level->space[o][p].flags, AVAILABLE);
                switch(trandom()%4)
                    {
                    case 0:
                        change_space(&this_level->space[m+1+trandom()%(k - 2)]
                          [n+1], S_OPEN_DOOR, this_level->race);
                    break;
                    case 1:
                        change_space(&this_level->space[m+1+trandom()%(k - 2)]
                          [n+l + 1], S_OPEN_DOOR, this_level->race);
                    break;
                    case 2:
                        change_space(&this_level->space[m+1]
                          [trandom()%(l - 2)+n+1], S_OPEN_DOOR,
                          this_level->race);
                    break;
                    case 3:
                        change_space(&this_level->space[m+ k + 1]
                          [trandom()%(l - 2)+n+1], S_OPEN_DOOR,
                          this_level->race);
                    break;
                    }
                my_success = 1;
                }
        break; /* T_BUILDING_SQUARE */
        case T_BUILDING_TURRETS: /* a castle-like building with four towers */
                                 /* rectangular room */
            k=5;
            while
                (trandom()%2)
                ++k;
            l=5;
            while
                (trandom()%2)
                ++l;
            m=trandom()%(MAX_X-k-2);
            n=trandom()%(MAX_Y-l-2);
            my_valid=1;
            for(o=m;o<m+k+2;++o)
                for(p=n;p<n+l+2;++p)
                    if (!(((o == m + 1) || (o == m + k)) && ((p == n + 1) ||
                      (p == n + l))))
                        my_valid=(my_valid &&
                        flag_check(this_level->space[o][p].flags, AVAILABLE));
            if (my_valid)
                {
                for(o=m+1;o<m+k+2;++o)
                    for(p=n+1;p<n+l+2;++p)
                        if (!((((o == m + 1) || (o == m + k)) && (p > n + 2) &&
                          (p < n + l - 2)) || (((p == n + 1) || (p == n + l))
                          && (o > m + 2) && (o < m + k - 2))))
                            {
                            change_space(&this_level->space[o][p], other,
                              this_level->race);
                            flag_reset(&this_level->space[o][p].flags,
                              CAN_OCCUPY);
                            }
                for(o=m;o<m+k+3;++o)
                    for(p=n;p<n+l+3;++p)
                        if (!(((o == m) || (o == m + k + 2)) && ((p == n) ||
                          (p == n + l + 2))))
                            flag_reset(&this_level->space[o][p].flags,
                              AVAILABLE);
                switch(trandom()%4)
                    {
                    case 0:
                        change_space(&this_level->space[m+trandom()%(k - 4) + 2]
                          [n + 1], S_OPEN_DOOR, this_level->race);
                    break;
                    case 1:
                        change_space(&this_level->space[m+trandom()%(k - 4) + 2]
                          [n+l - 1], S_OPEN_DOOR, this_level->race);
                    break;
                    case 2:
                        change_space(&this_level->space[m + 1]
                          [trandom()%(l - 4)+n + 2], S_OPEN_DOOR,
                          this_level->race);
                    break;
                    case 3:
                        change_space(&this_level->space[m+k - 1]
                          [trandom()%(l - 4)+n + 2], S_OPEN_DOOR,
                          this_level->race);
                    break;
                    }
                my_success = 1;
                }
        break; /* T_BUILDING_TURRETS */
        case T_CLOCKWORK_DOOR:
            if (other == LEVEL_MAZE)
                {
                switch(trandom() % 2)
                    {
                    case 0:
                        k = 1 + 2 * ((trandom() % MAX_X) / 2);
                        l = 2 * ((trandom() % MAX_X) / 2);
                    break;
                    case 1:
                        k = 2 * ((trandom() % MAX_X) / 2);
                        l = 1 + 2 * ((trandom() % MAX_X) / 2);
                    break;
                    }
                }
            else
                {
                k = trandom() % MAX_X;
                l = trandom() % MAX_Y;
                }
            if (flag_check(this_level->space[k][l].flags, AVAILABLE))
                change_space(&this_level->space[k][l], S_CLOCKWORK_DOOR,
                  this_level->race);
        break; /* T_CLOCKWORK_DOOR */
        case T_CLOSED_DOOR:
            if (other == LEVEL_MAZE)
                {
                switch(trandom() % 2)
                    {
                    case 0:
                        k = 1 + 2 * ((trandom() % MAX_X) / 2);
                        l = 2 * ((trandom() % MAX_X) / 2);
                    break;
                    case 1:
                        k = 2 * ((trandom() % MAX_X) / 2);
                        l = 1 + 2 * ((trandom() % MAX_X) / 2);
                    break;
                    }
                }
            else
                {
                k = trandom() % MAX_X;
                l = trandom() % MAX_Y;
                }
            if (flag_check(this_level->space[k][l].flags, AVAILABLE))
                change_space(&this_level->space[k][l], S_CLOSED_DOOR,
                this_level->race);
        break; /* T_CLOSED_DOOR */
        case T_DRY_TREE:
            l=k=1;
            m=trandom()%(MAX_X-k-2);
            n=trandom()%(MAX_Y-l-2);
            my_valid=1;
            for(o=m;o<m+k+2;++o)
                for(p=n;p<n+l+2;++p)
                    my_valid=(my_valid &&
                    flag_check(this_level->space[o][p].flags, AVAILABLE));
            if (my_valid)
                {
                flag_reset(&this_level->space[m + k + 1][n + l + 1].flags,
                  AVAILABLE);
                change_space(&this_level->space[m + k + 1][n + l + 1],
                  S_DRY_TREE, this_level->race);
                my_success = 1;
                }
            for(o=m+k-1;o<m+k+2;++o)
                for(p=n+l-1;p<n+l+2;++p)
                    flag_reset(&this_level->space[k][l].flags, AVAILABLE);
                    /* debug this later */
        break; /* T_DRY_TREE */
        case T_DUNGEON_ADD_ROOM:
            found_space = FALSE;
            for(i = 0; i < MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    found_space |= flag_check(this_level->space[i][j].flags,
                      CAN_OCCUPY);
            k = l = 2;
            while (trandom() % 2)
                ++k;
            while (trandom() % 2)
                ++l;
            m = 2 + trandom() % (MAX_X - k - 4);
            n = 2 + trandom() % (MAX_Y - l - 4);
            my_valid = TRUE;
            for(i = m - 2; i < m + k + 3; ++i)
                for(j = n - 2; j < n + l + 3; ++j)
                    my_valid &= (this_level->space[i][j].type == L_WALL);
            if (my_valid)
                {
                my_success = TRUE;
                for(i = m; i < m + k + 1; ++i)
                    for(j = n; j < n + k + 1; ++j)
                        {
                        change_space(&this_level->space[i][j], S_FLOOR,
                          this_level->race);
                        flag_set(&this_level->space[i][j].flags, AVAILABLE);
                        }
                if (found_space)
                    {
                    best_x = -1000;
                    best_y = -1000;
                    for(i = 0; i < MAX_X; ++i)
                        for(j = 0; j < MAX_Y; ++j)
                            if ((i < m) || (i > m + k + 1) || (j < n) || (j >
                              n + l + 1))
                                if ((distance(m + k / 2, n + l / 2, i, j) <
                                  distance(m + k / 2, n + l / 2, best_x,
                                  best_y)) &&
                                  (flag_check(this_level->space[i][j].flags,
                                  CAN_OCCUPY)))
                                    {
                                    best_x = i;
                                    best_y = j;
                                    }
                    o = m + k / 2;
                    p = n + l / 2;
                    assigned_space = FALSE;
                    assigned_first_door = FALSE;
                    while ((o != best_x) || (p != best_y))
                        {
                        if (trandom() % (abs(best_x - o) + abs(best_y - p)) <
                          abs(best_x - o))
                            o += (best_x - o) / (abs(best_x - o));
                        else
                            p += (best_y - p) / (abs(best_y - p));
                        if (this_level->space[o][p].type == L_WALL)
                            assigned_space = TRUE;
                        if (((assigned_space) && (!(assigned_first_door))) ||
                          (abs(best_x - o) + abs(best_y - p) == 1))
                            {
                            change_space(&this_level->space[o][p], other,
                              this_level->race);
                            assigned_first_door = TRUE;
                            }
                        else
                            change_space(&this_level->space[o][p], S_FLOOR,
                              this_level->race);
                        }
                    }
                }
        break;
        case T_DUNGEON_JOIN_COMPONENTS:
            my_valid = TRUE;
            for(i = 0; i < MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    if (flag_check(this_level->space[i][j].flags, CAN_OCCUPY))
                        {
                        mark_connected_component_can_occupy(this_level, i, j);
                        i = MAX_X;
                        j = MAX_Y;
                        }
            for(i = 0; i < MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    if ((flag_check(this_level->space[i][j].flags, CAN_OCCUPY))
                      && (!(flag_check(this_level->space[i][j].flags,
                      MARKED))))
                        {
                        i = MAX_X;
                        j = MAX_Y;
                        my_valid = FALSE;
                        }
            if (!(my_valid))
                {
                num_spaces = 0;
                for(i = 0; i < MAX_X; ++i)
                    for(j = 0; j < MAX_Y; ++j)
                        if ((flag_check(this_level->space[i][j].flags,
                          CAN_OCCUPY)) && (flag_check(this_level->
                          space[i][j].flags, MARKED)))
                            ++num_spaces;
                chosen = trandom() % num_spaces;
                num_spaces = 0;
                for(i = 0; i < MAX_X; ++i)
                    for(j = 0; j < MAX_Y; ++j)
                        if ((flag_check(this_level->space[i][j].flags,
                          CAN_OCCUPY)) && (flag_check(this_level->
                          space[i][j].flags, MARKED)))
                            {
                            if (num_spaces == chosen)
                                {
                                k = i;
                                l = j;
                                }
                            ++num_spaces;
                            }
                num_spaces = 0;
                for(i = 0; i < MAX_X; ++i)
                    for(j = 0; j < MAX_Y; ++j)
                        if ((flag_check(this_level->space[i][j].flags,
                          CAN_OCCUPY)) && (!(flag_check(this_level->
                          space[i][j].flags, MARKED))))
                            ++num_spaces;
                chosen = trandom() % num_spaces;
                num_spaces = 0;
                for(i = 0; i < MAX_X; ++i)
                    for(j = 0; j < MAX_Y; ++j)
                        if ((flag_check(this_level->space[i][j].flags,
                          CAN_OCCUPY)) && (!(flag_check(this_level->
                          space[i][j].flags, MARKED))))
                            {
                            if (num_spaces == chosen)
                                {
                                m = i;
                                n = j;
                                }
                            ++num_spaces;
                            }
                    while ((m != k) || (n != l))
                        {
                        if (trandom() % (abs(k - m) + abs(l - n)) <
                          abs(k - m))
                            o += (k - m) / (abs(k - m));
                        else
                            p += (l - n) / (abs(l - n));
                        change_space(&this_level->space[m][n], S_FLOOR,
                          this_level->race);
                        }
                my_success = TRUE;
                }
            unmark_level(this_level);
        break;
        case T_FOUNTAIN:
            k=1 + trandom()%2;
            l=k;
            m=trandom()%(MAX_X-k-2);
            n=trandom()%(MAX_Y-l-2);
            my_valid=1;
            for(o=m;o<m+k+2;++o)
                for(p=n;p<n+l+2;++p)
                    my_valid=(my_valid &&
                    flag_check(this_level->space[o][p].flags, AVAILABLE));
            if (my_valid)
                {
                for(o=m+1;o<m+k+1;++o)
                    for(p=n+1;p<n+l+1;++p)
                        change_space(&this_level->space[o][p], S_FOUNTAIN,
                          this_level->race);
                for(o=m;o<m+k+2;++o)
                    for(p=n;p<n+l+2;++p)
                        flag_reset(&this_level->space[o][p].flags, AVAILABLE);
                my_success = 1;
                }
        break; /* T_FOUNTAIN */
        case T_FRUIT_TREE:
            l=k=1;
            m=trandom()%(MAX_X-k-2);
            n=trandom()%(MAX_Y-l-2);
            my_valid=1;
            for(o=m;o<m+k+2;++o)
                for(p=n;p<n+l+2;++p)
                    my_valid=(my_valid &&
                    flag_check(this_level->space[o][p].flags, AVAILABLE));
            if (my_valid)
                {
                flag_reset(&this_level->space[m + k + 1][n + l + 1].flags,
                  AVAILABLE);
                change_space(&this_level->space[m + k + 1][n + l + 1],
                  S_FRUIT_TREE, this_level->race);
                my_success = 1;
                }
            for(o=m+k-1;o<m+k+2;++o)
                for(p=n+l-1;p<n+l+2;++p)
                    flag_reset(&this_level->space[k][l].flags, AVAILABLE);
                    /* debug this later */
        break; /* T_FRUIT_TREE */
        case T_GARDEN:
            l=k=5;
            m=trandom()%(MAX_X-k-2);
            n=trandom()%(MAX_Y-l-2);
            my_valid=1;
            for(o=m;o<m+k+2;++o)
                for(p=n;p<n+l+2;++p)
                    my_valid=(my_valid &&
                    flag_check(this_level->space[o][p].flags, AVAILABLE));
            if (my_valid)
                {
                my_selected = trandom() % NUM_GARDENS;
                for(o=m+1;o<m+k+1;++o)
                    for(p=n+1;p<n+l+1;++p)
                        {
                        change_space(&this_level->space[o][p],
                          gardens[my_selected][o - m - 1][p - n - 1],
                          this_level->race);
                        this_level->space[o][p].room_description = ST_A_GARDEN;
                        }
                for(o=m;o<m+k+2;++o)
                    for(p=n;p<n+l+2;++p)
                        flag_reset(&this_level->space[o][p].flags, AVAILABLE);
                my_success = 1;
                }
        break;
        case T_GREEN_TREE:
            l=k=1;
            m=trandom()%(MAX_X-k-2);
            n=trandom()%(MAX_Y-l-2);
            my_valid=1;
            for(o=m;o<m+k+2;++o)
                for(p=n;p<n+l+2;++p)
                    my_valid=(my_valid &&
                    flag_check(this_level->space[o][p].flags, AVAILABLE));
            if (my_valid)
                {
                flag_reset(&this_level->space[m + k + 1][n + l + 1].flags,
                  AVAILABLE);
                change_space(&this_level->space[m + k + 1][n + l + 1],
                  S_GREEN_TREE, this_level->race);
                my_success = 1;
                }
            for(o=m+k-1;o<m+k+2;++o)
                for(p=n+l-1;p<n+l+2;++p)
                    flag_reset(&this_level->space[k][l].flags, AVAILABLE);
                    /* debug this later */
        break; /* T_GREEN_TREE */
        case T_HEDGE:
            if (trandom()%2 > 0)
                {
                k=3;
                while
                    (trandom()%2)
                    ++k;
                l=1;
                }
            else
                {
                l=3;
                while
                    (trandom()%2)
                    ++l;
                k = 1;
                }
            m=trandom()%(MAX_X-k-2);
            n=trandom()%(MAX_Y-l-2);
            my_valid=1;
            for(o=m + 1;o<m+k+1;++o)
                for(p=n + 1;p<n+l+1;++p)
                    my_valid=(my_valid &&
                    flag_check(this_level->space[i][j].flags, AVAILABLE));
            if (my_valid)
                {
                for(o=m+1;o<m+k+1;++o)
                    for(p=n+1;p<n+l+1;++p)
                        change_space(&this_level->space[o][p], S_HEDGE,
                          this_level->race);
                for(o=m;o<m+k+2;++o)
                    for(p=n;p<n+l+2;++p)
                        flag_reset(&this_level->space[o][p].flags, AVAILABLE);
                my_success = 1;
                }
        break; /* T_HEDGE */
        case T_HIDDEN_MAZE_ENTRANCE:
            i = trandom() % MAX_X;
            j = trandom() % MAX_Y;
            my_success = FALSE;
            for(k = max(0, i - 1); k < min(MAX_X, i + 2); ++k)
                for(l = max(0, j - 1); l < min(MAX_Y, j + 2); ++l)
                    my_success |= (flag_check(this_level->space[k][l].flags,
                      CAN_OCCUPY));
            my_success &= (!(flag_check(this_level->space[k][l].flags,
                    CAN_OCCUPY)));
            if (my_success)
                {
                flag_set(&this_level->space[k][l].flags, HAS_SECRET);
                this_level->space[k][l].other = S_MAZE_ENTRANCE;
                }
        break; /* T_HIDDEN_MAZE_ENTRANCE */
        case T_LOCKED_DOOR:
            if (other == LEVEL_MAZE)
                {
                switch(trandom() % 2)
                    {
                    case 0:
                        k = 1 + 2 * ((trandom() % MAX_X) / 2);
                        l = 2 * ((trandom() % MAX_X) / 2);
                    break;
                    case 1:
                        k = 2 * ((trandom() % MAX_X) / 2);
                        l = 1 + 2 * ((trandom() % MAX_X) / 2);
                    break;
                    }
                }
            else
                {
                k = trandom() % MAX_X;
                l = trandom() % MAX_Y;
                }
            if (flag_check(this_level->space[k][l].flags, AVAILABLE))
                change_space(&this_level->space[k][l], S_LOCKED_DOOR,
                  this_level->race);
        break; /* T_LOCKED_DOOR */
        case T_MANSION_HORIZONTAL_WALL:
            my_valid = TRUE;
            i = 3 + trandom() % (MAX_X - 3);
            j = 3 + trandom() % (MAX_Y - 3);
            for(k = i - 1; k < i + 2; ++k)
                for(l = j - 1; l < j + 2; ++l)
                    my_valid &= (flag_check(this_level->space[k][l].flags,
                      AVAILABLE));
            if (my_valid)
                {
                my_continue = TRUE;
                for(k = i; ((my_continue) && (k < MAX_X)); ++k)
                    {
                    my_continue &= ((flag_check(this_level->space[k][j].flags,
                      AVAILABLE)) && (this_level->space[k][j].type ==
                      L_FLOOR));
                    if (!(my_continue))
                        my_valid &= (this_level->space[k][j].type == L_WALL);
                    m = k - 1;
                    }
                }
            if (my_valid)
                {
                my_continue = TRUE;
                for(k = i; ((my_continue) && (k >= 0)); --k)
                    {
                    my_continue &= ((flag_check(this_level->space[k][j].flags,
                      AVAILABLE)) && (this_level->space[k][j].type ==
                      L_FLOOR));
                    if (!(my_continue))
                        my_valid &= (this_level->space[k][j].type == L_WALL);
                    n = k + 1;
                    }
                }
            if (my_valid)
                {
                for(k = n; k <= m; ++k)
                    change_space(&this_level->space[k][j], S_WALL,
                      this_level->race);
                change_space(&this_level->space[n + trandom() % (m - n +
                  1)][j], S_CLOCKWORK_DOOR, this_level->race);
                change_space(&this_level->space[n + trandom() % (m - n +
                  1)][j], S_CLOCKWORK_DOOR, this_level->race);
                }
        break; /* T_MANSION_HORIZONTAL_WALL */
        case T_MANSION_VERTICAL_WALL:
            my_valid = TRUE;
            i = 3 + trandom() % (MAX_X - 6);
            j = 3 + trandom() % (MAX_Y - 6);
            for(k = i - 1; k < i + 2; ++k)
                for(l = j - 1; l < j + 2; ++l)
                    my_valid &= (flag_check(this_level->space[k][l].flags,
                      AVAILABLE));
            if (my_valid)
                {
                my_continue = TRUE;
                for(l = j; ((my_continue) && (l < MAX_Y)); ++l)
                    {
                    my_continue &= ((flag_check(this_level->space[i][l].flags,
                      AVAILABLE)) && (this_level->space[i][l].type ==
                      L_FLOOR));
                    if (!(my_continue))
                        my_valid &= (this_level->space[i][l].type == L_WALL);
                    m = l - 1;
                    }
                }
            if (my_valid)
                {
                my_continue = TRUE;
                for(l = j; ((my_continue) && (l >= 0)); --l)
                    {
                    my_continue &= ((flag_check(this_level->space[i][l].flags,
                      AVAILABLE)) && (this_level->space[i][l].type ==
                      L_FLOOR));
                    if (!(my_continue))
                        my_valid &= (this_level->space[i][l].type == L_WALL);
                    n = l + 1;
                    }
                }
            if (my_valid)
                {
                for(l = n; l <= m; ++l)
                    change_space(&this_level->space[i][l], S_WALL,
                      this_level->race);
                change_space(&this_level->space[i][n + trandom() % (m - n +
                  1)], S_CLOCKWORK_DOOR, this_level->race);
                change_space(&this_level->space[i][n + trandom() % (m - n +
                  1)], S_CLOCKWORK_DOOR, this_level->race);
                }
        break; /* T_MANSION_VERTICAL_WALL */
        case T_MAZE_JOIN_SPACES:
        /* Grow the maze by picking one enclosed square, and then adding to */
        /* the edges of the continuously connected area until it is the */
        /* whole maze */
            i = 1 + 2 * (trandom() % ((MAX_X/2) - 1));
            j = 1 + 2 * (trandom() % ((MAX_X/2) - 1));
            flag_set(&this_level->space[i][j].flags, MARKED);
            my_continue = 1;
            while (my_continue == 1)
                {
                for (i = 1; i <= 1 + (2 * ((MAX_X/2) - 1) ); i += 2)
                    for (j = 1; j < 1 + (2 * ((MAX_Y/2) - 1) ); j += 2)
                        if (((flag_check(this_level->space[i][j].flags,
                          MARKED)) ^
                          (flag_check(this_level->space[i][j + 2].flags,
                          MARKED))) && (trandom() % 2 == 0))
                            {
                            change_space(&this_level->space[i][j + 1], S_FLOOR,
                              this_level->race);
                            flag_set(&this_level->space[i][j].flags, MARKED);
                            flag_set(&this_level->space[i][j + 1].flags,
                              MARKED);
                            flag_set(&this_level->space[i][j + 2].flags,
                              MARKED);
                            }
                for (i = 1; i < 1 + (2 * ((MAX_X/2) - 1) ); i += 2)
                    for (j = 1; j <= 1 + (2 * ((MAX_Y/2) - 1) ); j += 2)
                        if (((flag_check(this_level->space[i][j].flags,
                          MARKED)) ^
                          (flag_check(this_level->space[i + 2][j].flags,
                          MARKED))) && (trandom() % 2 == 0))
                            {
                            change_space(&this_level->space[i + 1][j], S_FLOOR,
                              this_level->race);
                            flag_set(&this_level->space[i][j].flags, MARKED);
                            flag_set(&this_level->space[i + 1][j].flags,
                              MARKED);
                            flag_set(&this_level->space[i + 2][j].flags,
                              MARKED);
                            }
                my_continue = 0;
                for (i = 1; i <= (2 * ((MAX_X/2) - 1) ); i += 2)
                    for (j = 1; j <= (2 * ((MAX_Y/2) - 1) ); j += 2)
                        if (!(flag_check(this_level->space[i][j].flags,
                          MARKED)))
                            my_continue = 1;
                }
            unmark_level(this_level);
        break; /* T_MAZE_JOIN_SPACES */
        case T_OPEN_DOOR:
            if (other == LEVEL_MAZE)
                {
                switch(trandom() % 2)
                    {
                    case 0:
                        k = 1 + 2 * ((trandom() % MAX_X) / 2);
                        l = 2 * ((trandom() % MAX_X) / 2);
                    break;
                    case 1:
                        k = 2 * ((trandom() % MAX_X) / 2);
                        l = 1 + 2 * ((trandom() % MAX_X) / 2);
                    break;
                    }
                }
            else
                {
                k = trandom() % MAX_X;
                l = trandom() % MAX_Y;
                }
            if (flag_check(this_level->space[k][l].flags, AVAILABLE))
                change_space(&this_level->space[k][l], S_OPEN_DOOR,
                  this_level->race);
        break; /* T_OPEN_DOOR */
        case T_POOL:
            k=3;
            while
                (trandom()%2)
                ++k;
            l=5*(k)*(k);
            m=k + 1 + trandom()%(MAX_X-2*k-3);
            n=k + 1 + trandom()%(MAX_Y-2*k-3);
            mark_pool(this_level, m, n, k, l);
            my_valid=1;
            for(o=0;o<MAX_X;++o)
                for(p=0;p<MAX_Y;++p)
                    if ((0 <= o) && (o <= MAX_X) && (0 <= p) && (p <= MAX_Y))
                        if (flag_check(this_level->space[o][p].flags, MARKED))
                            {
                            my_valid=(my_valid &&
                              flag_check(this_level->space[o][p].flags,
                              AVAILABLE));
                            if ((0 < o) && (o < MAX_X) && (0 < p) && (p <
                              MAX_Y))
                                for(i=o-1; i<o+2;++i)
                                    for(j=p-1; j < p - 2; ++j)
                                        my_valid=(my_valid &&
                                          flag_check(this_level->space[i][j]
                                            .flags, AVAILABLE));
                            }
            if (my_valid)
                {
                mark_pool(this_level, m, n, k, l);
                for(o=0;o<MAX_X;++o)
                    for(p=0;p<MAX_Y;++p)
                        if (flag_check(this_level->space[o][p].flags, MARKED))
                            {
                            change_space(&this_level->space[o][p], S_WATER,
                              this_level->race);
                            for(i=o-1;i<o+2;++i)
                                for(j=p-1;j<p+2;++j)
                                    {
                                    flag_reset(&this_level->space[o][p].flags,
                                      AVAILABLE|MARKED);
                                    /* make the boundary shallow */
                                    if (!(flag_check(this_level->space[i][j].
                                      flags, IS_WATER)))
                                        {
                                        flag_set(&this_level->space[o][p].flags,
                                          SHALLOW);
                                        this_level->space[o][p].property =
                                          P_SHALLOW_WATER;
                                        }
                                    }
                            }
                my_success=1;
                }
                unmark_level(this_level);
        break; /* T_POOL */
        case T_ROOM_CIRCULAR:
            k=2;
            while
                (trandom()%2)
                ++k;
            l=k;
            m=k + 1 + trandom()%(MAX_X-2*k-3);
            n=l + 1 + trandom()%(MAX_Y-2*l-3);
            mark_circle(this_level, m, n, k + 1);
            my_valid=1;
            for(o=m - k - 1;o<m+k+2;++o)
                for(p=n - l - 1;p<n+l+2;++p)
                    if ((0 <= o) && (o <= MAX_X) && (0 <= p) && (p <= MAX_Y))
                        if (flag_check(this_level->space[o][p].flags, MARKED))
                            my_valid=(my_valid && (this_level->space[o][p].type
                              == L_WALL));
            unmark_level(this_level);
            if (my_valid)
                {
                mark_circle(this_level, m, n, k);
                i = m;
                j = n;
                find_boundary(this_level, &i, &j);
                change_space(&this_level->space[i][j], S_OPEN_DOOR,
                  this_level->race);
                for(o=m-k;o<m+k+1;++o)
                    for(p=n-l;p<n+l+1;++p)
                        if ((0 <= o) && (o <= MAX_X) && (0 <= p) && (p <=
                          MAX_Y))
                            if (flag_check(this_level->space[o][p].flags,
                              MARKED))
                                {
                                change_space(&this_level->space[o][p], other,
                                  this_level->race);
                                /* Keep people from walking into buildings,
                                  even buildings woven out of living trees */
                                flag_reset(&this_level->space[o][p].flags,
                                  CAN_OCCUPY);
                                for(i=o-1;i<o+2;++i)
                                    for(j=p-1;j<p+2;++j)
                                        flag_reset(
                                          &this_level->space[o][p].flags,
                                          AVAILABLE);
                                }
                my_success=1;
                }
        break; /* T_ROOM_CIRCULAR */
        case T_ROOM_RECTANGULAR:
            k=2;
            while
                (trandom()%2)
                ++k;
            l=2;
            while
                (trandom()%2)
                ++l;
            m=trandom()%(MAX_X-k-2);
            n=trandom()%(MAX_Y-l-2);
            my_valid=1;
            for(o=m;o<m+k+3;++o)
                for(p=n;p<n+l+3;++p)
                    my_valid=(my_valid &&
                    (this_level->space[o][p].type == L_WALL));
            if (my_valid)
                {
                for(o=m+1;o<m+k+2;++o)
                    for(p=n+1;p<n+l+2;++p)
                        {
                        change_space(&this_level->space[o][p], other,
                          this_level->race);
                        flag_reset(&this_level->space[o][p].flags, CAN_OCCUPY);
                        }
                for(o=m;o<m+k+2;++o)
                    for(p=n;p<n+l+2;++p)
                        flag_reset(&this_level->space[o][p].flags, AVAILABLE);
                switch(trandom()%4)
                    {
                    case 0:
                        change_space(&this_level->space[m+trandom()%k+1][n+1],
                          S_OPEN_DOOR, this_level->race);
                    break;
                    case 1:
                        change_space(&this_level->space[m+trandom()%k+1][n+l+1],
                          S_OPEN_DOOR, this_level->race);
                    break;
                    case 2:
                        change_space(&this_level->space[m+1][trandom()%l+n+1],
                          S_OPEN_DOOR, this_level->race);
                    break;
                    case 3:
                        change_space(&this_level->space[m+k+1]
                          [trandom()%l+n+1], S_OPEN_DOOR, this_level->race);
                    break;
                    }
                my_success = 1;
                }
        break; /* T_ROOM_RECTANGULAR */
        case T_ROOM_ROUNDED: /* corners rounded off an otherwise */
                                 /* rectangular room */
            k=5;
            while
                (trandom()%2)
                ++k;
            l=5;
            while
                (trandom()%2)
                ++l;
            m=trandom()%(MAX_X-k-2);
            n=trandom()%(MAX_Y-l-2);
            my_valid=1;
            for(o=m;o<m+k+2;++o)
                for(p=n;p<n+l+2;++p)
                    if (!(((o == m + 1) || (o == m + k)) && ((p == n + 1) ||
                      (p == n + l))))
                        my_valid=(my_valid &&
                          (this_level->space[o][p].type == L_WALL));
            if (my_valid)
                {
                for(o=m+1;o<m+k+2;++o)
                    for(p=n+1;p<n+l+2;++p)
                        if (!(((o == m + 1) || (o == m + k + 1)) && ((p == n +
                          1) || (p == n + l + 1))))
                            {
                            change_space(&this_level->space[o][p], other,
                              this_level->race);
                            flag_reset(&this_level->space[o][p].flags,
                              CAN_OCCUPY);
                            }
                for(o=m;o<m+k+3;++o)
                    for(p=n;p<n+l+3;++p)
                        if (!(((o == m) || (o == m + k + 1)) && ((p == n) || (p
                          == n + l + 1))))
                            flag_reset(&this_level->space[o][p].flags,
                              AVAILABLE);
                switch(trandom()%4)
                    {
                    case 0:
                        change_space(&this_level->space[m+trandom()%(k - 4) + 2]
                          [n + 1], S_OPEN_DOOR, this_level->race);
                    break;
                    case 1:
                        change_space(&this_level->space[m+trandom()%(k - 4) + 2]
                          [n+l + 1], S_OPEN_DOOR, this_level->race);
                    break;
                    case 2:
                        change_space(&this_level->space[m + 1]
                          [trandom()%(l - 4)+n + 2], S_OPEN_DOOR,
                          this_level->race);
                    break;
                    case 3:
                        change_space(&this_level->space[m+k + 1]
                          [trandom()%(l - 4)+n + 2], S_OPEN_DOOR,
                          this_level->race);
                    break;
                    }
                my_success = 1;
                }
        break; /* T_ROOM_ROUNDED */
        case T_ROOM_SQUARE: /* corners rounded off an otherwise */
                                 /* rectangular room */
            k=3;
            while
                (trandom()%2)
                ++k;
            l = k;
            m=trandom()%(MAX_X-k-2);
            n=trandom()%(MAX_Y-l-2);
            my_valid=1;
            for(o=m;o<m+k+3;++o)
                for(p=n;p<n+l+3;++p)
                        my_valid=(my_valid &&
                        (this_level->space[o][p].type == L_WALL));
            if (my_valid)
                {
                for(o=m + 1;o<m+k+2;++o)
                    for(p=n + 1;p<n+l+2;++p)
                        {
                        change_space(&this_level->space[o][p], other,
                          this_level->race);
                        flag_reset(&this_level->space[o][p].flags, CAN_OCCUPY);
                        }
                for(o=m;o<m+k+2;++o)
                    for(p=n;p<n+l+2;++p)
                        flag_reset(&this_level->space[o][p].flags, AVAILABLE);
                switch(trandom()%4)
                    {
                    case 0:
                        change_space(&this_level->space[m+1+trandom()%(k - 2)]
                          [n+1], S_OPEN_DOOR, this_level->race);
                    break;
                    case 1:
                        change_space(&this_level->space[m+1+trandom()%(k - 2)]
                          [n+l + 1], S_OPEN_DOOR, this_level->race);
                    break;
                    case 2:
                        change_space(&this_level->space[m+1]
                          [trandom()%(l - 2)+n+1], S_OPEN_DOOR,
                          this_level->race);
                    break;
                    case 3:
                        change_space(&this_level->space[m+ k + 1]
                          [trandom()%(l - 2)+n+1], S_OPEN_DOOR,
                          this_level->race);
                    break;
                    }
                my_success = 1;
                }
        break; /* T_ROOM_SQUARE */
        case T_SEASIDE: /* This should be done before anything else.  */
                        /* A variation on the theme of a staggering drunk. */
            i = trandom() % 4;
            for (j = trandom() % 4; j == i; j = trandom()%4)
                my_noop();
            switch(i)
                {
                case 0:
                    m = 1 + trandom() % (MAX_X - 2);
                    n = 0;
                break;
                case 1:
                    m = 1 + trandom() % (MAX_X - 2);
                    n = MAX_X - 1;
                break;
                case 2:
                    m = 0;
                    n = 1 + trandom() % (MAX_Y - 2);
                break;
                case 3:
                    m = MAX_Y - 1;
                    n = 1 + trandom() % (MAX_Y - 2);
                break;
                }
            switch(j)
                {
                case 0:
                    o = 1 + trandom() % (MAX_X - 2);
                    p = 0;
                break;
                case 1:
                    o = 1 + trandom() % (MAX_X - 2);
                    p = MAX_X - 1;
                break;
                case 2:
                    o = 0;
                    p = 1 + trandom() % (MAX_Y - 2);
                break;
                case 3:
                    o = MAX_Y - 1;
                    p = 1 + trandom() % (MAX_Y - 2);
                break;
                }
            i = m;
            j = n;
            while (!((i == o) && (j == p)))
                {
                if (trandom() % (abs(i - o) + abs(j - p)) < abs(i - o))
                    i += (o - i)/abs(o - i);
                else
                    j += (p - j)/abs(p - j);
                change_space(&this_level->space[i][j], S_WATER,
                  this_level->race);
                flag_set(&this_level->space[i][j].flags, SHALLOW|SALT_WATER);
                this_level->space[i][j].room_description = ST_THE_OCEAN;
                flag_reset(&this_level->space[i][j].flags, AVAILABLE);
                }
            i = (MAX_X/2) - 1;
            j = (MAX_Y/2) - 1;
            if (this_level->space[i][j].type == L_WATER)
                i = j = 0;
            mark_connected_component(this_level, i, j);
            for(i=0;i<MAX_X;++i)
                for(j=0;j<MAX_Y;++j)
                    if ((flag_check(this_level->space[i][j].flags, AVAILABLE))
                      && (!(flag_check(this_level->space[i][j].flags, MARKED))))
                        {
                        change_space(&this_level->space[i][j], S_WATER,
                        this_level->race);
                        flag_set(&this_level->space[i][j].flags, SALT_WATER);
                        }
            unmark_level(this_level);
        break; /* T_SEASIDE */
        case T_SECRET_PASSAGE:
            if (other == LEVEL_MAZE)
                {
                switch(trandom() % 2)
                    {
                    case 0:
                        k = 1 + 2 * ((trandom() % MAX_X) / 2);
                        l = 2 * ((trandom() % MAX_X) / 2);
                    break;
                    case 1:
                        k = 2 * ((trandom() % MAX_X) / 2);
                        l = 1 + 2 * ((trandom() % MAX_X) / 2);
                    break;
                    }
                }
            else
                {
                k = trandom() % MAX_X;
                l = trandom() % MAX_Y;
                }
            if (flag_check(this_level->space[k][l].flags, AVAILABLE))
                change_space(&this_level->space[k][l], S_SECRET_PASSAGE,
                this_level->race);
        break; /* T_SECRET_PASSAGE */
        case T_STAIRWELL_DOWN: /* Create the down stairwell */
            num_spaces = curr_spaces = 0;
            for(k=0;k<MAX_X;++k)
                for(l=0;l<MAX_Y;++l)
                    if (flag_check(this_level->space[k][l].flags, AVAILABLE))
                        ++num_spaces;
            if (num_spaces > 0)
                {
                my_selected=trandom()%num_spaces;
                for(k=0;k<MAX_X;++k)
                    for(l=0;l<MAX_Y;++l)
                        if (flag_check(this_level->space[k][l].flags,
                          AVAILABLE))
                            {
                            if (curr_spaces==my_selected)
                                {
                                change_space(&this_level->space[k][l],
                                  S_STAIRWELL_DOWN, this_level->race);
                                this_level->space[k][l].special->level_type =
                                  other;
                                if (other == LEVEL_COUNTRYSIDE)
                                    {
                                    this_level->space[k][l].special->
                                      takes_player_to = countryside;
                                    this_level->space[k][l].special->x =
                                      game_variables->countryside_player_x;
                                    this_level->space[k][l].special->y =
                                      game_variables->countryside_player_y;
                                    }
                                for(o=k - 1;o<k+2;++o)
                                for(p=l - 1;p<l+2;++p)
                                    if ((0 <= o) && (o < MAX_X) && (0 <= p) &&
                                      (p < MAX_Y))
                                        flag_reset(&this_level->space[o][p]
                                          .flags, AVAILABLE);
                                }
                            ++curr_spaces;
                            }
                my_success=1;
                }
        break; /* T_STAIRWELL_DOWN */
        case T_STAIRWELL_UP: /* Create the up stairwell */
            num_spaces = curr_spaces =0;
            for(k=0;k<MAX_X;++k)
                for(l=0;l<MAX_Y;++l)
                    if (flag_check(this_level->space[k][l].flags, AVAILABLE))
                        ++num_spaces;
            if (num_spaces > 0)
                {
                my_selected=trandom()%num_spaces;
                for(k=0;k<MAX_X;++k)
                    for(l=0;l<MAX_Y;++l)
                        if (flag_check(this_level->space[k][l].flags,
                          AVAILABLE))
                            {
                            if (curr_spaces==my_selected)
                                {
                                change_space(&this_level->space[k][l],
                                  S_STAIRWELL_UP, this_level->race);
                                this_level->space[k][l].special->level_type =
                                  other;
                                if (other == LEVEL_COUNTRYSIDE)
                                    {
                                    this_level->space[k][l].special->
                                      takes_player_to = countryside;
                                    this_level->space[k][l].special->x =
                                      game_variables->countryside_player_x;
                                    this_level->space[k][l].special->y =
                                      game_variables->countryside_player_y;
                                    }
                                for(o=max(0, k - 1);o<min(MAX_X, k+2);++o)
                                    for(p=max(0, l - 1);p<min(MAX_X, l+2);++p)
                                        flag_reset(&this_level->space[o][p].
                                          flags, AVAILABLE);
                                }
                            ++curr_spaces;
                            }
                my_success=1;
                }
        break; /* T_STAIRWELL_UP */
        case T_STATUE:
            l=k=1;
            m=trandom()%(MAX_X-k-2);
            n=trandom()%(MAX_Y-l-2);
            my_valid=1;
            for(o=m;o<m+k+2;++o)
                for(p=n;p<n+l+2;++p)
                    my_valid=(my_valid &&
                    flag_check(this_level->space[o][p].flags, AVAILABLE));
            if (my_valid)
                {
                flag_reset(&this_level->space[o][p].flags, AVAILABLE);
                change_space(&this_level->space[m + k + 1][n + l + 1],
                  S_STATUE, this_level->race);
                my_success = 1;
                }
            for(o=m+k-1;o<m+k+2;++o)
                for(p=n+l-1;p<n+l+2;++p)
                    flag_reset(&this_level->space[k][l].flags, AVAILABLE);
        break; /* T_STATUE */
        case T_STREAM: /* This should be done before anything else.  */
                        /* A variation on the theme of a staggering drunk. */
            i = trandom() % 4;
            for (j = trandom() % 4; j == i; j = trandom()%4)
                my_noop();
            switch(i)
                {
                case 0:
                    m = 1 + trandom() % (MAX_X - 2);
                    n = 0;
                break;
                case 1:
                    m = 1 + trandom() % (MAX_X - 2);
                    n = MAX_X - 1;
                break;
                case 2:
                    m = 0;
                    n = 1 + trandom() % (MAX_Y - 2);
                break;
                case 3:
                    m = MAX_Y - 1;
                    n = 1 + trandom() % (MAX_Y - 2);
                break;
                }
            switch(j)
                {
                case 0:
                    o = 1 + trandom() % (MAX_X - 2);
                    p = 0;
                break;
                case 1:
                    o = 1 + trandom() % (MAX_X - 2);
                    p = MAX_X - 1;
                break;
                case 2:
                    o = 0;
                    p = 1 + trandom() % (MAX_Y - 2);
                break;
                case 3:
                    o = MAX_Y - 1;
                    p = 1 + trandom() % (MAX_Y - 2);
                break;
                }
            i = m;
            j = n;
            /* We have the conditional because we check for a stream after */
            /* possibly putting a land on the seaside.  */
            if (!(this_level->space[i][j].type == L_WATER))
                {
                change_space(&this_level->space[i][j], S_WATER,
                  this_level->race);
                flag_set(&this_level->space[i][j].flags, SHALLOW);
                this_level->space[i][j].space_description = ST_A_STREAM;
                if ((i > 0) && (i < MAX_X) && (j > 0) && (j < MAX_Y))
                    for(k=i-1;k<i+2;++k)
                        for(l=j-1;l<j+2;++l)
                            flag_reset(&this_level->space[k][l].flags,
                              AVAILABLE);
                }
            while (!((i == o) && (j == p)))
                {
                if (trandom() % (abs(i - o) + abs(j - p)) < abs(i - o))
                    i += (o - i)/abs(o - i);
                else
                    j += (p - j)/abs(p - j);
                if (!(this_level->space[i][j].type == L_WATER))
                    {
                    change_space(&this_level->space[i][j], S_WATER,
                      this_level->race);
                    flag_set(&this_level->space[i][j].flags, SHALLOW);
                    this_level->space[i][j].space_description = ST_A_STREAM;
                    if ((i > 0) && (i < MAX_X) && (j > 0) && (j < MAX_Y))
                        for(k=i-1;k<i+2;++k)
                            for(l=j-1;l<j+2;++l)
                                flag_reset(&this_level->space[k][l].flags,
                                  AVAILABLE);
                    }
                }
        break; /* T_STREAM */
        case T_WINDOW:
            l=k=1;
            m=trandom()%(MAX_X-k-2);
            n=trandom()%(MAX_Y-l-2);
            my_valid=0;
            for(o=-1;o<2;++o)
                for(p=-1;p<2;++p)
                    my_valid = (my_valid || ((this_level->space[m + k + o]
                      [n + l + p].type == L_FLOOR) && (this_level->space
                      [m + k - o][n + l - p].type == L_FLOOR)));
            my_valid = (my_valid && (this_level->space[m + k][n + l].type ==
              L_WALL));
            for(o=m + 1;o<m+k+1;++o)
                for(p=n + 1;p<n+l+1;++p)
                    my_valid=(my_valid &&
                    flag_check(this_level->space[i][j].flags, AVAILABLE));
            if (my_valid)
                {
                flag_reset(&this_level->space[m + k + 1][n + l + 1].flags,
                  AVAILABLE);
                change_space(&this_level->space[m + k + 1][n + l + 1],
                  S_STATUE, this_level->race);
                my_success = 1;
                }
        break; /* T_WINDOW */
        }       
    return my_success;
    }

#ifdef CODE_SEGMENT
#pragma segment LEVEL3_C_B
#endif

struct small_creature *make_random_creature(long ctype)
    {
    long type, age_class, gender, garbage, temp;
    struct small_creature *result;
    result = NULL;
    if (ctype<=LAST_ANIMAL)
        {
        temp=trandom()%10;
        if (temp<3)
            age_class=CHILD;
        else if (temp<6)
            age_class=YOUNG_ADULT;
        else if (temp<8)
            age_class=MIDDLE_AGED;
        else if (temp<9)
            age_class=OLD;
        else
            age_class=ANCIENT;
        gender=trandom()%2;
        result=creature_create(ctype, age_class, gender);
        }
    else 
        {
        switch(ctype)
            {
            case JEC_ENCOUNTER:
                random_select(jec_encounter, &type, &garbage);
                result = make_random_creature(type);
            break;
            case NORKRIN_ENCOUNTER:
                random_select(norkrin_encounter, &type, &garbage);
                result = make_random_creature(type);
            break;
            case SHAL_ENCOUNTER:
                random_select(shal_encounter, &type, &garbage);
                result = make_random_creature(type);
            break;
            case TUZ_ENCOUNTER:
                random_select(tuz_encounter, &type, &garbage);
                result = make_random_creature(type);
            break;
            case URVANOVESTILLI_ENCOUNTER:
                random_select(urvanovestilli_encounter, &type, &garbage);
                result = make_random_creature(type);
            break;
            case YEDIDIA_ENCOUNTER:
                random_select(yedidia_encounter, &type, &garbage);
                result = make_random_creature(type);
            break;
            case SEA_ENCOUNTER:
                random_select(sea_encounter, &type, &garbage);
                result = make_random_creature(type);
            break;
            case FISHING_ENCOUNTER:
                random_select(fishing_encounter, &type, &garbage);
                result = make_random_creature(type);
            break;
            case LEVEL_1_ENCOUNTER:
                random_select(level_1_encounter, &type, &garbage);
                result = make_random_creature(type);
            break;
            case LEVEL_2_ENCOUNTER:
                random_select(level_2_encounter, &type, &garbage);
                result = make_random_creature(type);
            break;
            case LEVEL_3_ENCOUNTER:
                random_select(level_3_encounter, &type, &garbage);
                result = make_random_creature(type);
            break;
            case LEVEL_4_ENCOUNTER:
                random_select(level_4_encounter, &type, &garbage);
                result = make_random_creature(type);
            break;
            case LEVEL_5_ENCOUNTER:
                random_select(level_5_encounter, &type, &garbage);
                result = make_random_creature(type);
            break;
            case LAND_SPECIAL_ENCOUNTER:
                random_select(land_special_encounter, &type, &garbage);
                result = make_random_creature(type);
            break;
            case SEA_SPECIAL_ENCOUNTER:
                random_select(sea_special_encounter, &type, &garbage);
                result = make_random_creature(type);
            break;
            default:
#ifdef MAC_68k
                pmsg("Error: monster creation type out of range.");
#else
                perror("Error: monster creation type out of range.");
#endif
            break;
            }
        }
    return result;
    }
