/***************************************************************************/
/* The Minstrel's Song is (c) 1998-2005 by Jonathan Hayward.  See the file */
/* lib/copyrigh.txt for a copyright and license statement.                 */
/***************************************************************************/

#include "tms.h"
#include <stdio.h>
#ifndef GTK
#include <curses.h>
#endif

#ifdef CODE_SEGMENT
#pragma segment SKILLS_C
#endif

char *tracking_choice[2] =
    {
    "Search for a specific creature.",
    "Look for anything."
    };

void use_skill(struct small_creature *c, long which)
    {
    long can_use, i, j, hostile_found;
    struct clist *cltraverse;
    struct ilist *iltraverse;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
#ifndef GTK
    which=skill_parse();
#endif
    can_use = 1;
    if (which == NONE)
        {
        can_use = 0;
        }
    else if (!(flag_check(skill_info[which].flags, GENERALLY_USABLE)))
        {
        pmsg("That skill is not used in this manner.");
        can_use = 0;
        }
    else if (flag_check(skill_info[which].flags, NEEDS_ITEM))
        {
        can_use = 0;
        if (!(c->extended_creature == NULL))
            for(iltraverse = c->extended_creature->wear_wield; (!(iltraverse == NULL));
              iltraverse = iltraverse->next)
        if (iltraverse->current != NULL)
                if ((orig_items[iltraverse->current->identity]->skill_enable[0]
                  == which)
                  || (orig_items[iltraverse->current->identity]->skill_enable[1]
                  == which))
                    can_use = 1;
        }
    else if (flag_check(skill_info[which].flags, NO_HOSTILE))
        {
        hostile_found = FALSE;
        for(cltraverse = current_level->creatures; (!(cltraverse == NULL));
          cltraverse = cltraverse->next)
        if (cltraverse->current != NULL)
            if (is_hostile(cltraverse->current, player))
                {
                hostile_found = TRUE;
                }
        if (hostile_found)
            {
#ifdef GTK
            can_use &= tms_confirm("That skill may take some time to use, and there may be hostile creatures\naround!  Use skill anyway (and risk being attacked)?");
#else
            pmsg("That skill may take some time to use, and there may be hostile creatures");
            nocap_pmsg("around!  Use skill anyway (and risk being attacked)?");
            can_use &= confirm_silent();
#endif
            }
        }
    if ((flag_check(skill_info[which].flags, NEEDS_SPECIAL)) && (can_use == 1))
        {
        switch(which)
            {
            case CHEMISTRY:
                can_use = flag_check(current_level->space[c->x][c->y].flags,
                  CAN_CHEM);
            break;
            case FISHING:
                can_use = 0;
                for(i = max(0, c->x - 1); i < min(MAX_X, c->x + 2); ++i)
                    for(j = max(0, c->y - 1); j < min(MAX_Y, c->y + 2); ++j)
                        can_use = (can_use || (current_level->space[i][j].type
                          == L_WATER));
            break;
            case GEOGRAPHY:
                can_use = (current_level == countryside);
            break;
            case HERBALISM:
                can_use = flag_check(current_level->space[c->x][c->y].flags,
                  CAN_HERB);
            break;
            case NAVIGATION:
                can_use = (current_level == countryside);
            break;
            case TINKERING:
                can_use = flag_check(current_level->space[c->x][c->y].flags,
                  CAN_TINKER);
            break;
            case TRACKING:
                can_use = ((current_level == countryside) &&
                  (countryside->space[c->x][c->y].special->
                  animal_encounter_type != NONE));
            break;
            default:
                pmsg("Error.  Out of range skill has NEEDS_SPECIAL set.");
                can_use = 0;
            break;
            }
        if (can_use == 0)
            pmsg("You cannot use that skill here.");
        }
    if (can_use == 1)
        {
        skill_effect(c, which);
        }
    }

void skill_effect(struct small_creature *c, long skill)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    switch(skill)
        {
        case ANIMAL_HANDLING:
            pacify(c, loc_creature(c));
        break;
        case APPRAISAL:
            appraise(c, get_item(c, EXTRACT_RECURSIVE));
        break;
        case CHEMISTRY:
            make_item(c, skill);
        break;
        case FISHING:
            go_fishing(c);
        break;
        case GEOGRAPHY:
            geography(c);
        break;
        case HERBALISM:
            make_item(c, skill);
        break;
        case JUMPING:
            jump(c);
        break;
        case MEDICINE:
            medicine(c, loc_creature(c));
        break;
        case MUSIC_FLUTE:
            music(c, skill);
        break;
        case MUSIC_HARP:
            music(c, skill);
        break;
        case MUSIC_LUTE:
            music(c, skill);
        break;
        case MUSIC_ORGAN:
            music(c, skill);
        break;
        case MUSIC_SINGING:
            music(c, skill);
        break;
        case NAVIGATION:
            navigate(c);
        break;
        case PICK_LOCKS:
            pick_locks(c);
        break;
        case REPAIR:
            repair(c, get_item(c, EXTRACT_RECURSIVE));
        break;
        case STEALTH:
            attempt_hide(c);
        break;
        case TECHNOLOGY_IDENTIFICATION:
            attempt_identify(c, get_item(c, EXTRACT_RECURSIVE));
        break;
        case TINKERING:
            make_item(c, skill);
        break;
        case TRACKING:
            track(c);
        break;
        case WRESTLING:
            wrestle(c, loc_creature(c));
        break;
        }
    }

void pacify(struct small_creature *c1, struct small_creature *c2)
    {
    long success;
    char message[256], creature_name[256];
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c1);
    check_pointer((struct pointer_struct *) c2);
#endif
    if (c2 != NULL)
        {
        if (!(c2->name == NULL))
            {
            strncpy(creature_name, c2->name->text, 255);
            creature_name[255] = '\0';
            }
        else if (!(c2->extended_creature == NULL))
            {
            strncpy(creature_name, c2->extended_creature->title, 255);
            creature_name[255] = '\0';
            if ((creature_name[0] >= 'a') && (creature_name[0] <= 'z'))
                creature_name[0] += 'A' - 'a';
            }
        else
#ifdef HAS_SNPRINTF
            snprintf(creature_name, 255, "The %s",
              master_creatures[c2->identity]->species);
#else
            sprintf(creature_name, "The %s", master_creatures[c2->identity]->
              species);
#endif
        creature_name[255] = '\0';
            if ((!(c1->extended_creature == NULL)) && (!(c2 == NULL)))
                {
                if (flag_check(c2->flags, HUMAN))
                    {
                    success = skill_check(c1, c1->extended_creature->skills->av
                      [PERSUASION], master_creatures[c2->identity]->
                      persuade_diff + ONE_NOTCH,
                      SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
                    take_time(c1, skill_info[PERSUASION].time);
                    }
                else
                    {
                    success = skill_check(c1,
                      c1->extended_creature->skills->av[ANIMAL_HANDLING],
                      master_creatures[c2->identity]->persuade_diff,
                      SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
                    take_time(c1, skill_info[ANIMAL_HANDLING].time);
                    if (success > 0)
                        flag_set(&game_variables->quests_completed, 
                          TAMED_ANIMAL);
                    }
            if (success < 0)
                {
                if (c1 == player)
                    {
#ifdef HAS_SNPRINTF
                    snprintf(message, 255, "Um, problem.  %s seems quite annoyed...",
                      creature_name);
#else
                    sprintf(message, "Um, problem.  %s seems quite annoyed...",
                      creature_name);
#endif
                    message[255] = '\0';
                    pmsg(message);
                    }
                make_hostile(c2, c1);
                }
            else if (success == 0)
                {
                if (c1 == player)
                    pmsg("Nothing happens.");
                }
            else if (success == 1)
                {
                if (is_hostile(c2, c1))
                    {
                    reset_hostility(c2, c1);
                    if (c1 == player)
                        {
#ifdef HAS_SNPRINTF
                        snprintf(message, 255, "%s stops attacking you.",
                          creature_name);
#else
                        sprintf(message, "%s stops attacking you.",
                          creature_name);
#endif
                        message[255] = '\0';
                        pmsg(message);
                        }
                    }
                else
                    {
                    make_follow(c2, c1);
                    if (c1 == player)
                        {
#ifdef HAS_SNPRINTF
                        snprintf(message, 255, "%s starts following you.",
                          creature_name);
#else
                        sprintf(message, "%s starts following you.",
                          creature_name);
#endif
                        message[255] = '\0';
                        pmsg(message);
                        }
                    }
                }
            else /* Critical success */
                {
                reset_hostility(c2, c1);
                make_follow(c2, c1);
                if (c1 == player)
                    {
#ifdef HAS_SNPRINTF
                    snprintf(message, 255, "%s starts following you.", creature_name);
#else
                    sprintf(message, "%s starts following you.", creature_name);
#endif
                    message[255] = '\0';
                    pmsg(message);
                    }
                }
            }
        }
    }

void appraise(struct small_creature *c, struct small_item *i)
    {
    char message[256], temp[256];
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
    check_pointer((struct pointer_struct *) i);
#endif
    if (i != NULL)
        {
        if (flag_check(i->flags2, APPRAISED))
            {
            money_string(i->value, temp);
#ifdef HAS_SNPRINTF
            snprintf(message, 255, "It is worth %s.", temp);
#else
            sprintf(message, "It is worth %s.", temp);
#endif
            message[255] = '\0';
            pmsg(message);
            }
        else if (flag_check(i->flags2, APPRAISE_FAILED))
            pmsg("You have already unsuccessfully attempted to appraise that.");
        else
            {
            if (c->extended_creature != NULL)
                {
                if (skill_check(c, c->extended_creature->skills->av[APPRAISAL],
                  orig_items[i->identity]->cdce_diff,
                  SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE))
                    {
                    money_string(i->value, temp);
#ifdef HAS_SNPRINTF
                    snprintf(message, 255, "It is worth %s.", temp);
#else
                    sprintf(message, "It is worth %s.", temp);
#endif
                    message[255] = '\0';
                    pmsg(message);
                    flag_set(&i->flags2, APPRAISED);
                    }
                else
                    {
                    pmsg("You can't tell what it's worth.");
                    flag_set(&i->flags2, APPRAISE_FAILED);
                    }
                }
            }
        }
    }

void attempt_hide(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (flag_check(c->flags, HAS_LIGHT_SOURCE))
        {
        if (c == player)
            pmsg("You attempt to hide...");
        flag_reset(&c->flags, HIDDEN);
        if (c->extended_creature != NULL)
            {
            if (skill_check(c, c->extended_creature->skills->av[STEALTH], DIFFICULT,
              SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE))
                flag_set(&c->flags, HIDDEN);
            }
        else
            {
            if (skill_check(c, VERY_DIFFICULT, DIFFICULT,
              SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE))
                flag_set(&c->flags, HIDDEN);
            }
        }
    else
        if (c == player)
            pmsg("You must extinguish your light before attempting any kind of stealth.");
    }

void attempt_identify(struct small_creature *c, struct small_item *i)
    {
    char temp[256], temp2[256];
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
    check_pointer((struct pointer_struct *) i);
#endif
    if (i != NULL)
        {
        if (!(flag_check(i->flags2, IDENTIFY_FAILED)))
            {
            if (c->extended_creature != NULL)
                {
                if (skill_check(c,
                  c->extended_creature->skills->av[TECHNOLOGY_IDENTIFICATION],
                  orig_items[i->identity]->cdce_diff - ONE_NOTCH,
                  SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE) > 0)
                    {
                    identify(i);
                    item_desc(i, temp);
                    sprintf(temp2, "Identified as a %s.", temp);
                    pmsg(temp2);
                    }
                else
                    {
                    pmsg("You can't figure out what it is.");
                    flag_set(&i->flags2, IDENTIFY_FAILED);
                    }
                }
            }
        else
            pmsg("You have already attempted (unsuccessfully) to identify that.");
        }
    }

void geography(struct small_creature *c)
    {
    long success, i, j;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c->extended_creature != NULL)
        {
        success = skill_check(c, c->extended_creature->skills->av[GEOGRAPHY], EASY,
          SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
        if (success > 0)
            for(i = 0; i < MAX_X; ++i)
                for(j = 0; j < MAX_Y; ++j)
                    if (((c->x - i) * (c->x - i) + (c->y - j) * (c->y - j)) <=
                      (2 * success * success))
                        {
                        flag_set(&current_level->space[i][j].flags, KNOWN);
                        if (((c->x - i) * (c->x - i) + (c->y - j) * (c->y - j))
                          <= (success * success))
                            discover_secret(i, j);
                        }
        update_window();
        }
    }

void go_fishing(struct small_creature *c)
    {
    long success, i;
    struct small_creature *fish;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c->extended_creature != NULL)
        {
        success = skill_check(c, c->extended_creature->skills->av[FISHING],
          DIFFICULT, SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
        if (success > 0)
            {
            pmsg("You catch something!");
            for(i = 0; i < success; ++i)
                {
                fish = make_random_creature(FISHING_ENCOUNTER);
                make_hostile(fish, c);
                no_hostile_add_creature(current_level, fish, c->x, c->y);
                }
            }
        else
            pmsg("You didn't catch anything.");
        }
    }

void guess_actions(struct small_creature *c, struct small_creature *ctarget)
    {
    long success;
    char message[256], creature_name[256];
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
    check_pointer((struct pointer_struct *) ctarget);
#endif
    if (ctarget != NULL)
        {
        success = skill_check(c, c->extended_creature->skills->av[GUESS_ACTIONS],
          MODERATE, SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
        find_title(c, creature_name);
        if (success <= 0)
            pmsg("You can't tell anything.");
        else if (success == 1)
            {
            if (ctarget->follow != NULL)
#ifdef HAS_SNPRINTF
                snprintf(message, 255, "%s is following something.", creature_name);
#else
                sprintf(message, "%s is following something.", creature_name);
#endif
            else
#ifdef HAS_SNPRINTF
                snprintf(message, 255, "%s is not following anything.",
                  creature_name);
#else
                sprintf(message, "%s is not following anything.",
                  creature_name);
#endif
            message[255] = '\0';
            pmsg(message);
            if (ctarget->hostile != NULL)
#ifdef HAS_SNPRINTF
                snprintf(message, 255, "%s is hostile to something.", creature_name);
#else
                sprintf(message, "%s is hostile to something.", creature_name);
#endif
            else
#ifdef HAS_SNPRINTF
                snprintf(message, 255, "%s is not hostile to anything.",
                  creature_name);
#else
                sprintf(message, "%s is not hostile to anything.",
                  creature_name);
#endif
            message[255] = '\0';
            pmsg(message);
            if (ctarget->extended_creature != NULL)
                {
                if (ctarget->extended_creature->wear_wield != NULL)
#ifdef HAS_SNPRINTF
                    snprintf(message, 255, "%s is using something.", creature_name);
#else
                    sprintf(message, "%s is using something.", creature_name);
#endif
                else
#ifdef HAS_SNPRINTF
                    snprintf(message, 255, "%s is not using anything.",
                      creature_name);
#else
                    sprintf(message, "%s is not using anything.",
                      creature_name);
#endif
                message[255] = '\0';
                pmsg(message);
                if (ctarget->extended_creature->wear_wield != NULL)
#ifdef HAS_SNPRINTF
                    snprintf(message, 255, "%s is carrying something.",
                      creature_name);
#else
                    sprintf(message, "%s is carrying something.",
                      creature_name);
#endif
                else
#ifdef HAS_SNPRINTF
                    snprintf(message, 255, "%s is not carrying anything.",
                      creature_name);
#else
                    sprintf(message, "%s is not carrying anything.",
                      creature_name);
#endif
                message[255] = '\0';
                pmsg(message);
                }
            }
        else
            {
            if (ctarget->follow != NULL)
                {
#ifdef HAS_SNPRINTF
                snprintf(message, 255, "%s is following:", creature_name);
#else
                sprintf(message, "%s is following:", creature_name);
#endif
                message[255] = '\0';
                pmsg(message);
                pmsg("");
                print_clist(ctarget->follow, 0);
                }
            else
                {
#ifdef HAS_SNPRINTF
                snprintf(message, 255, "%s is not following anything.",
                  creature_name);
#else
                sprintf(message, "%s is not following anything.",
                  creature_name);
#endif
                message[255] = '\0';
                pmsg(message);
                }
            pmsg("");
            if (ctarget->hostile != NULL)
                {
#ifdef HAS_SNPRINTF
                snprintf(message, 255, "%s is hostile to:", creature_name);
#else
                sprintf(message, "%s is hostile to:", creature_name);
#endif
                message[255] = '\0';
                pmsg(message);
                pmsg("");
                print_clist(ctarget->hostile, 0);
                }
            else
                {
#ifdef HAS_SNPRINTF
                snprintf(message, 255, "%s is not hostile to anything.",
                  creature_name);
#else
                sprintf(message, "%s is not hostile to anything.",
                  creature_name);
#endif
                message[255] = '\0';
                pmsg(message);
                }
            pmsg("");
            if (ctarget->extended_creature != NULL)
                {
                if (ctarget != player)
#ifdef HAS_SNPRINTF
                    snprintf(message, 255, "%s's inventory:",
                      creature_name);
#else
                    sprintf(message, "%s's inventory:",
                      creature_name);
#endif
                else
#ifdef HAS_SNPRINTF
                    snprintf(message, 255, "Your inventory:");
#else
                    sprintf(message, "Your inventory:");
#endif
                message[255] = '\0';
                pmsg(message);
                morewait();
                print_inventory(ctarget);
                }
            }
        }
    }

void hear_noises(struct small_creature *c)
    {
    char creature_title[256], message[256];
    long success;
    struct clist *traverse;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c->extended_creature != NULL)
        {
        /* for experience's sake.  */
        skill_check(c, c->extended_creature->skills->av[HEAR_NOISES], DIFFICULT,
          SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
        for(traverse = current_level->creatures; traverse != NULL; traverse =
          traverse->next)
            {
        if (traverse->current != NULL)
        {
            success = skill_check(c, c->extended_creature->skills->av[HEAR_NOISES],
              distance(c->x, c->y, traverse->current->x, traverse->current->y)
              + ((!(clear_path(c->x, c->y, traverse->current->x,
              traverse->current->y))) * ONE_NOTCH), 0);
            if (success > 0)
                {
                if (success == 1)
                    pmsg("You hear something.");
                else
                    {
                    find_title(traverse->current, creature_title);
#ifdef HAS_SNPRINTF
                    snprintf(message, 255, "You hear %s!", creature_title);
#else
                    sprintf(message, "You hear %s!", creature_title);
#endif
                    message[255] = '\0';
                    pmsg(message);
                    if (success > 2)
                        if (traverse->current->hostile != NULL)
                            {
#ifdef HAS_SNPRINTF
                            snprintf(message, 255, "%s sounds agigated!",
                              creature_title);
#else
                            sprintf(message, "%s sounds agigated!",
                              creature_title);
#endif
                            message[255] = '\0';
                            }
                    }
                }
        }
            }
        }
    }

void jump(struct small_creature *c)
    {
    long x, y, difficulty, success;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (current_level != countryside)
        {
        pmsg("Jump where?");
        if ((loc_xy(c, &x, &y)) && (c->extended_creature != NULL))
            {
            if (clear_path(c->x, c->y, x, y))
                {
                difficulty = distance(c->x, c->y, x, y);
                success = skill_check(c, c->extended_creature->skills->av[JUMPING],
                  difficulty * current_level->scale / 3,
                  SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
                if (success < 0)
                    {
                    pmsg("Oops; tumbled and fell.");
                    if (creature_damage(c, 0 - success,
                      CREATURE_DAMAGE_WAKE_UP, 0 - success))
                        creature_death(c);
                    else
                        {
                        move_creature(c, (x - c->x)/(1 - success),
                          (y - c->y)/(1 - success));
                        }
                    }
                else if (success == 1)
                    {
                    pmsg("Boing!");
                    move_creature(c, x - c->x, y - c->y);
                    }
                else
                    {
                    pmsg("You jump with finesse!");
                    move_creature(c, x - c->x, y - c->y);
                    }
                }
            else
                pmsg("The way is obstructed.");
            }
        }
    else
        pmsg("You can't jump that way in the countryside!");
    }

void make_item(struct small_creature *c, long skill)
    {
    long which, success;
    struct small_item *result;
    struct ilist *this_ilist;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    result = NULL;
    if (!(c->extended_creature == NULL))
        {
        switch(skill)
            {
            case CHEMISTRY:
                pmsg("Which chemical do you wish to make?");
                which = item_parse(CAN_CHEM);
            break;
            case HERBALISM:
                pmsg("Which herb or herbal product do you wish to search for or make?");
                which = item_parse(CAN_HERB);
            break;
            case TINKERING:
                pmsg("Which device do you wish to make?");
                which = item_parse(CAN_TINKER);
            break;
            default:
                pmsg("Error; make_item called with inappropriate skill.");
                which = NONE;
            break;
            }
        if (which == NONE)
            pmsg("Aborted.");
        else
            {
            success = skill_check(c, c->extended_creature->skills->av[skill],
              orig_items[which]->cdce_diff,
              SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
            if (success < 0)
                {
                pmsg("You couldn't make anything.");
                result = NULL;
                }
            else if (success == 0)
                {
                pmsg("You managed a mediocre job of it.");
                result = generate_item(current_level, which);
                small_item_make_normal(result);
                small_item_make_poor(result);
                flag_set(&result->flags, IDENTIFIED);
                }
            else if (success == 1)
                {
                pmsg("Created!");
                result = generate_item(current_level, which);
                small_item_make_normal(result);
                flag_set(&result->flags, IDENTIFIED);
                }
            else if (success > 1)
                {
                pmsg("A masterpiece!");
                result = generate_item(current_level, which);
                small_item_make_normal(result);
                small_item_make_fine(result);
                flag_set(&result->flags, IDENTIFIED);
                }
            if (!(result == NULL))
                {
                this_ilist = (struct ilist *) calloc(1, sizeof(struct ilist));
                this_ilist->me = this_ilist;
                this_ilist->current = result;
                this_ilist->next = current_level->space[c->x][c->y].items;
                current_level->space[c->x][c->y].items = this_ilist;
                pickup(c);
                }
            pass_time(skill_info[skill].time);
            }
        }
    }

void medicine(struct small_creature *c, struct small_creature *ctarget)
    {
    long success, wound;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
    check_pointer((struct pointer_struct *) ctarget);
#endif
    if (!(c->extended_creature == NULL))
        {
        if (!(ctarget->extended_creature == NULL))
            wound = ctarget->extended_creature->stati->time[STATUS_HEALABLE];
        else
            wound = ctarget->wounded;
        if (wound == 0)
            pmsg("Your medical skills can't do any good here.");
        else
            {
            success = min(skill_check(c, c->extended_creature->skills->av[MEDICINE],
              DIFFICULT, SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE), wound);
            if (success < 0)
                pmsg("Oops.");
            else if (success == 0)
                pmsg("Nothing happens.");
            else if (success == 1)
                pmsg("The injuries seem better.");
            else
                pmsg("The injuries seem much better.");
            if (!(ctarget->extended_creature == NULL))
                {
                ctarget->extended_creature->stati->time[STATUS_HEALABLE] -= success;
                ctarget->extended_creature->stati->time[STATUS_WOUNDED] -= success;
                calc_ready(ctarget);
                }
            else
                {
                ctarget->att += 3 * success;
                ctarget->def += 2 * success;
                ctarget->dmg += success;
                ctarget->hv += success;
                ctarget->sp += success;
                ctarget->wounded -= success;
                }
            take_time(c, skill_info[MEDICINE].time);
            }
        }
    }

void music(struct small_creature *c, long skill_id)
    {
    char message[256], creature_name[256];
    long success;
    struct clist *traverse;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (!(c->extended_creature == NULL))
        {
        if (flag_check(current_level->flags, IS_CITY))
            {
            success = skill_check(c, c->extended_creature->skills->av[skill_id],
              MODERATE, SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
            if (success < 0)
                {
                pmsg("Boo, boo!");
                pmsg("You are given a free shower of less-than-fresh vegetables");
                status_add(c, STATUS_STINKY, 1);
                }
            else if (success == 0)
                {
                pmsg("Nobody seems to take much notice of you.");
                }
            else
                {
                status_add(c, STATUS_PLEASING, success);
                pmsg("Your music draws a crowd.  Appreciative citizens throw you coins.");
                if (current_level->race == URVANOVESTILLI)
                    ++success;
                switch(success)
                    {
                    case 1:
                        c->extended_creature->cu += 1 + trandom() % 10 + trandom() % 10;
                    break;
                    case 2:
                        c->extended_creature->cu += 1 + trandom() % 10 + trandom() % 10;
                        c->extended_creature->si += 1 + trandom() % 10 + trandom() % 10;
                    break;
                    case 3:
                        c->extended_creature->cu += 1 + trandom() % 10 + trandom() % 10;
                        c->extended_creature->si += 1 + trandom() % 10 + trandom() % 10;
                        c->extended_creature->el += 1 + trandom() % 10 + trandom() % 10;
                    break;
                    case 4:
                        c->extended_creature->cu += 1 + trandom() % 10 + trandom() % 10;
                        c->extended_creature->si += 1 + trandom() % 10 + trandom() % 10;
                        c->extended_creature->el += 1 + trandom() % 10 + trandom() % 10;
                        c->extended_creature->au += 1 + trandom() % 10 + trandom() % 10;
                    break;
                    default:
                        c->extended_creature->cu += 1 + trandom() % 100 + trandom() %
                          100;
                        c->extended_creature->si += 1 + trandom() % 100 + trandom() %
                          100;
                        c->extended_creature->el += 1 + trandom() % 100 + trandom() %
                          100;
                        c->extended_creature->au += 1 + trandom() % 100 + trandom() %
                          100;
                    break;
                    }
                calc_ready(c);
                }
            }
        else /* Music soothes the savage beast */
            {
            pmsg("The music flows.");
            success = skill_check(c, c->extended_creature->skills->av[skill_id],
              MODERATE, SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
            status_add(c, STATUS_PLEASING, success);
            for(traverse = current_level->creatures; (!(traverse == NULL));
              traverse = traverse->next)
        if (traverse->current != NULL)
                if ((!(traverse->current == player)) &&
                  (!(is_following(traverse->current, player))))
                    {
                    success = skill_check(c, c->extended_creature->skills->av[skill_id],
                      master_creatures[traverse->current->identity]->
                      persuade_diff + ONE_NOTCH,
                      SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
                if (!(traverse->current->name == NULL))
                    {
                    strncpy(creature_name, traverse->current->name->text, 255);
                    creature_name[255] = '\0';
                    }
                else if (!(traverse->current->extended_creature == NULL))
                    {
                    strncpy(creature_name, traverse->current->extended_creature->title, 255);
                    creature_name[255] = '\0';
                    if ((creature_name[0] >= 'a') && (creature_name[0] <= 'z'))
                    creature_name[0] += 'A' - 'a';
                    }
                else
#ifdef HAS_SNPRINTF
                    snprintf(creature_name, 255, "The %s",
                      master_creatures[traverse->current->identity]->species);
#else
                    sprintf(creature_name, "The %s",
                      master_creatures[traverse->current->identity]->species);
#endif
                    creature_name[255] = '\0';
                    if (success == 1)
                        {
                        if (is_hostile(traverse->current, c))
                            {
                            reset_hostility(traverse->current, c);
                            if (c == player)
                                if (can_see(c, traverse->current->x,
                                  traverse->current->y))
                                    {
#ifdef HAS_SNPRINTF
                                    snprintf(message, 255, "%s stops attacking you.",
                                      creature_name);
#else
                                    sprintf(message, "%s stops attacking you.",
                                      creature_name);
#endif
                                    message[255] = '\0';
                                    pmsg(message);
                                    }
                            }
                        else
                            {
                            make_follow(traverse->current, c);
                            if (c == player)
                                if (can_see(c, traverse->current->x,
                                  traverse->current->y))
                                    {
#ifdef HAS_SNPRINTF
                                    snprintf(message, 255, "%s starts following you.",
                                      creature_name);
#else
                                    sprintf(message, "%s starts following you.",
                                      creature_name);
#endif
                                    message[255] = '\0';
                                    pmsg(message);
                                    }
                            }
                        }
                    else if (success > 1) /* Critical success */
                        {
                        reset_hostility(traverse->current, c);
                        make_follow(traverse->current, c);
                        if (c == player)
                            if (can_see(c, traverse->current->x,
                              traverse->current->y))
                                {
#ifdef HAS_SNPRINTF
                                snprintf(message, 255, "%s starts following you.",
                                  creature_name);
#else
                                sprintf(message, "%s starts following you.",
                                  creature_name);
#endif
                                message[255] = '\0';
                                pmsg(message);
                                }
                        }
                    }
            }
        pass_time(skill_info[skill_id].time);
        }
    }

void navigate(struct small_creature *c)
    {
    long success;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (status_check(c, STATUS_LOST > 0))
        {
        if (c->extended_creature != NULL)
            {
            success = skill_check(c, c->extended_creature->skills->av[NAVIGATION],
              DIFFICULT, SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
            status_add(c, STATUS_LOST, 0 - success);
            if (status_check(c, STATUS_LOST) == 0)
                pmsg("You find your way!");
            else
                pmsg("You're still lost.");
            }
        }
    else
        pmsg("You are not lost.");
    }

void pick_locks(struct small_creature *c)
    {
    long x, y;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    pmsg("Pick lock on (d)oor, or on (i)tem in your possession?");
    /* ## */
    switch(tgetc())
        {
        case 'd':
            pmsg("Where is the door?");
            if (loc_xy(c, &x, &y))
                {
                if (current_level->space[x][y].type == L_DOOR)
                    {
                    if (flag_check(current_level->space[x][y].flags,
                      DOOR_LOCKED))
                        {
                        if (c->extended_creature != NULL)
                            {
                            if (skill_check(c, c->extended_creature->skills->av
                              [PICK_LOCKS], DIFFICULT,
                              SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE) >
                              0)
                                {
                                pmsg("The lock clicks open.");
                                gain_experience(c, 10);
                                flag_reset(&current_level->space[x][y].flags,
                                  DOOR_LOCKED);
                                }
                            else
                                pmsg("You can't seem to pick the lock.");
                            }
                        }
                    else
                        pmsg("The door isn't locked!");
                    }
                else
                    if (current_level->space[x][y].type == L_CLOCKWORK_DOOR)
                        pmsg("You can't use lock picks on a clockwork door.");
                    else
                        pmsg("That isn't a door.");
                }
        break;
        case 'i':
        break;
        default:
            pmsg("Aborted.");
        break;
        }
    }

void repair(struct small_creature *c, struct small_item *i)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
    check_pointer((struct pointer_struct *) i);
#endif
    if ((c->extended_creature != NULL) && (i != NULL))
        {
        if (flag_check(i->flags, BROKEN))
            {
            if (!(flag_check(i->flags2, I_REPAIR_FAILED)))
                {
                if (skill_check(c, c->extended_creature->skills->av[REPAIR],
                orig_items[i->identity]->cdce_diff,
                SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE))
                    {
                    pmsg("Fixed!");
                    flag_reset(&i->flags, BROKEN);
                    }
                else
                    {
                    pmsg("You can't seem to fix it.");
                    flag_set(&i->flags2, I_REPAIR_FAILED);
                    }
                }
            else
                pmsg("You already tried, and can't fix it!");
            }
        else
            pmsg("It isn't broken!");
        }
    }

void smell_creature(struct small_creature *c, long diff_adjust)
    {
    char creature_title[256], message[256];
    long success;
    struct clist *traverse;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c->extended_creature != NULL)
        {
        /* for experience's sake.  */
        skill_check(c, c->extended_creature->skills->av[HEAR_NOISES], diff_adjust,
          SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
        for(traverse = current_level->creatures; traverse != NULL; traverse =
          traverse->next)
        if (traverse->current != NULL)
            {
            success = skill_check(c, c->extended_creature->skills->av[HEAR_NOISES],
              distance(c->x, c->y, traverse->current->x, traverse->current->y)
              / 4 + ((!(clear_path(c->x, c->y, traverse->current->x,
              traverse->current->y))) * ONE_NOTCH), 0);
            if (success > 0)
                {
                if (success == 1)
                    pmsg("You smell something.");
                else
                    {
                    find_title(traverse->current, creature_title);
#ifdef HAS_SNPRINTF
                    snprintf(message, 255, "You smell %s!", creature_title);
#else
                    sprintf(message, "You smell %s!", creature_title);
#endif
                    message[255] = '\0';
                    pmsg(message);
                    }
                }
            }
        }
    }

void track(struct small_creature *c)
    {
    char choice;
    long difficulty, encounter_type, success;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (current_level == countryside)
        {
        difficulty = DIFFICULT;
        if (current_level->space[c->x][c->y].special != NULL)
            {
            if (current_level->space[c->x][c->y].special->
              animal_encounter_frequency == NONE)
                {
                pmsg("There are no animals to be found here.  Perhaps it would be wiser to do your");
                nocap_pmsg("hunting away from a city...");
                }
            else
                {
                if (current_level->space[c->x][c->y].special->
                  animal_encounter_frequency >= 70)
                    difficulty += ONE_NOTCH;
                if (current_level->space[c->x][c->y].special->
                  animal_encounter_frequency <= 10)
                    difficulty -= ONE_NOTCH;
                pass_time(HOUR);
#ifdef GTK
                choice = menu_choice("Do you want to track down anything specific?", tracking_choice, 2);
                if (choice == 0)
                    choice = 't';
                else if (choice == 1)
                    choice = 'a';
                switch(choice)
#else
                pmsg("Try to (t)rack down a specific species of creature, or search for (a)nything?");
                switch(tgetc())
#endif
                    {
                    case 't':
                    encounter_type = creature_parse();
                    if (encounter_type != NONE)
                        {
                        if (current_level->space[c->x][c->y].special != NULL)
                            {
                            if (!(found_in_distribution(encounter_type,
                              current_level->space[c->x][c->y].special->
                              animal_encounter_type, DISTRIBUTION_CREATURE)))
                                difficulty += ONE_NOTCH / 4;
                            }
                        else
                            {
                            if (!(found_in_distribution(encounter_type,
                              current_level->animal_encounter_type,
                              DISTRIBUTION_CREATURE)))
                                difficulty += ONE_NOTCH / 4;
                            }
                        if (flag_check(
                          master_creatures[encounter_type]->flags, UNIQUE))
                            difficulty += ONE_NOTCH / 4;
                        if (encounter_type == KRAKEN)
                            difficulty -= 2 * ONE_NOTCH / 4;
                        }
                    break;
                    case 'a':
                        if (current_level->space[c->x][c->y].special != NULL)
                            encounter_type =
                              current_level->space[c->x][c->y].special->
                              animal_encounter_type;
                        else
                            {
                            pmsg("You can't find anything here.");
                            encounter_type = NONE;
                            }
                    break;
                    default:
                        pmsg("Aborted.");
                        encounter_type = NONE;
                    break;
                    }
                if (encounter_type != NONE)
                    {
                    if (c->extended_creature != NULL)
                        success = skill_check(c,
                          c->extended_creature->skills->av[TRACKING], difficulty,
                          SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
                    else
                        success = skill_check(c, VERY_DIFFICULT, difficulty,
                          SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
                    if (success <= 0)
                        pmsg("You couldn't track anything down.");
                    else
                        encounter(encounter_type, success);
                    }
                }
            }
        }
    else
        pmsg("You need to be in the countryside to track down creatures.");
    }

void wrestle(struct small_creature *c1, struct small_creature *c2)
    {
    long success;
    char c1_title[256], c2_title[256], message[256];
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c1);
    check_pointer((struct pointer_struct *) c2);
#endif
    find_title(c1, c1_title);
    find_title(c2, c2_title);
    if ((c1->extended_creature != NULL) && (c2->extended_creature != NULL))
        {
        success = skill_check(c1, c1->extended_creature->skills->av[WRESTLING],
          c2->extended_creature->skills->av[WRESTLING],
          SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
        if (success < 0)
#ifdef HAS_SNPRINTF
            snprintf(message, 255, "%s easily pins %s!", c2_title, c1_title);
#else
            sprintf(message, "%s easily pins %s!", c2_title, c1_title);
#endif
        else if (success == 0)
#ifdef HAS_SNPRINTF
            snprintf(message, 255, "%s pins %s.", c2_title, c1_title);
#else
            sprintf(message, "%s pins %s.", c2_title, c1_title);
#endif
        else if (success == 1)
#ifdef HAS_SNPRINTF
            snprintf(message, 255, "%s pins %s.", c1_title, c2_title);
#else
            sprintf(message, "%s pins %s.", c1_title, c2_title);
#endif
        else
#ifdef HAS_SNPRINTF
            snprintf(message, 255, "%s easily pins %s!", c1_title, c2_title);
#else
            sprintf(message, "%s easily pins %s!", c1_title, c2_title);
#endif
        message[255] = '\0';
        pmsg(message);
        }
    }
